-- "superclass" for buildings
local Building = {}


function Building:create( data )
	local entity = {}
	
	-- entity.entityType = 0
	entity.entityCategory = ENTITYCATEGORY.building
	
	entity.body = {
		x = data.x or 0,
		y = data.y or 0,
		z = 5,
		sizeX = 125,	-- ?
		sizeY = 125,
	}
	
	-- also store the tile index (avoid to calculate them over and over again)
	entity.xi = data.xi
	entity.yi = data.yi
	
	-- range in full tiles
	entity.explorationRange = 4
	
	entity.maxHealth = 500
	
	-- entity.image = Images["building_area_1"]
	entity.rendererLayer = "entities"
	
	-- blocking info in mini tiles
	-- for now each building can only occupy one full tile
	entity.blocking = {
		{0,0,0,0,0},
		{0,1,1,1,0},
		{0,1,1,1,0},
		{0,1,1,1,0},
		{0,0,0,0,0},
	}
	
	entity.health = Health:create( entity, entity.maxHealth )
	
	-- when the unit recently lost or gained health, draw the healthbar for a few seconds
	entity.timeSinceLastHealthChange = 0
	
	----------------------------------------------------------------------------------------------------
	
	function entity:getBlocking()
		return self.blocking
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- range in full tiles
	function entity:getExplorationRange()
		return self.explorationRange
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:load( id )
		self.id = id
		
		-- Gameloop:add( self )
		-- Renderer:add( self, "entities" )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:getPosition()
		return self.body.x, self.body.y
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- set currently selected - true / false
	function entity:setSelected( selected )
		self.selected = selected
		self.animationTick = 0
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- called from game when a hurt event occured
	function entity:onHurt( damage )
		
	end
	
	
	function entity:onHealthChanged()
		self.timeSinceLastHealthChange = 0
		
		if self.health:getHealth() <= 0 then
			-- TODO: collapse
			-- the resources spent to create this entity will be lost
			EntityManager:remove( self )
		end
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:tick( dt )
		
		if self.selected then
			self.animationTick = self.animationTick + dt
		end
		
		self.timeSinceLastHealthChange = self.timeSinceLastHealthChange + dt
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:destroy()
		Gameloop:remove( self )
		-- Renderer:remove( self, "entities" )
		self = nil
	end
	
	
	return entity
end

return Building
