local BuildingArea = {}
local lg = love.graphics


function BuildingArea:create( data )
	local entity = {}
	
	entity.entityType = ENTITIES.buildingArea
	entity.entityCategory = ENTITYCATEGORY.buildingArea
	
	entity.body = {
		x = data.x or 0,
		y = data.y or 0,
		sizeX = 125,	-- ?
		sizeY = 125,
	}
	
	-- also store the tile index (avoid to calculate them over and over again)
	entity.xi = data.xi
	entity.yi = data.yi
	
	entity.rendererLayer = "ground_entities"
	entity.image = Images["building_area_1"]
	
	----------------------------------------------------------------------------------------------------
	
	function entity:load( id )
		self.id = id
		
		Gameloop:add( self )
		Renderer:add( self, self.rendererLayer )
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- set currently selected - true / false
	function entity:setSelected( selected )
		self.selected = selected
		self.animationTick = 0
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:tick( dt )
		
		if self.selected then
			self.animationTick = self.animationTick + dt
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:draw()
		local body = self.body
		local x, y = convertToDimetric( body.x, body.y )
		local imgData = self.image
		local ox = imgData.ox or 0
		local oy = imgData.oy or 0
		
		if self.selected then
			-- a simple but decent looking "animation"; value goes from 0.6 to 1.0
			local v = 0.2 * math.sin( 3 * self.animationTick ) + 0.8
			-- lg.setColor( 0.8, 0.8, v )
			lg.setColor( v, v, 1 )
		end
		lg.draw( imgData.image, x, y, 0, 1, 1, ox, oy )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:destroy()
		Gameloop:remove( self )
		Renderer:remove( self, self.rendererLayer )
		self = nil
	end
	
	
	return entity
end

return BuildingArea
