local ConstructionFence = {}
local lg = love.graphics


function ConstructionFence:create( data )
	local entity = {}
	
	entity.entityCategory = ENTITYCATEGORY.buildingOther
	entity.entityType = ENTITIES.constructionFence
	
	entity.body = {
		-- shape?
		x = data.x or 0,
		y = data.y or 0,
		z = 5,
		sizeX = 25,	-- ?
		sizeY = 25,
	}
	
	-- reference (entity ID) to the associated construction site
	entity.constructionSiteId = data.constructionSiteId
	-- index 1 to 4
	entity.constructionSitePositionIndex = data.constructionSitePositionIndex
	
	entity.image = Images["construction_fence_" .. entity.constructionSitePositionIndex]
	entity.rendererLayer = "entities"
	
	
	----------------------------------------------------------------------------------------------------
	
	function entity:load( id )
		self.id = id
		
		Gameloop:add( self )
		Renderer:add( self, self.rendererLayer )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:getPosition()
		return self.body.x, self.body.y
	end
	
	
	function entity:setPosition( x, y )
		self.body.x = x
		self.body.y = y
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- cannot be picked up if carried already or placed at construction site
	function entity:canBePickedUp()
		local carrier = EntityManager:getEntityById( self.carrierEntityId )
		if carrier or self.isPlacedAtConstructionSite then
			return false
		end
		
		return true
	end
	
	-- worker (RR)
	function entity:setCarrierEntity( entity )
		if entity then
			self.carrierEntityId = entity.id
		else
			self.carrierEntityId = nil
		end
		-- print( "construction fence: setCarrierEntity", entity )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:tick( dt )
		
		-- if the construction site is gone (building canceled or completed) remove the fence
		local constructionSite = EntityManager:getEntityById( self.constructionSiteId )
		if not constructionSite then
			print( "construction fence removed - site is gone" )
			self:destroy()
			return
		end
		
		local carrier = EntityManager:getEntityById( self.carrierEntityId )
		if carrier then
			local x, y = carrier:getPosition()
			-- TODO: fetch correct draw offset depending on the carrier entity
			self:setPosition( x+1, y+1 )
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:draw()
		local body = self.body
		local x, y = convertToDimetric( body.x, body.y )
		
		local imgData = self.image
		local ox = imgData.ox or 0
		local oy = imgData.oy or 0
		
		local scale = self.isPlacedAtConstructionSite and 0.6 or 0.3
		
		lg.draw( imgData.image, x, y, 0, scale, scale, ox, oy )
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:destroy()
		-- if a worker carries this fence, cancel his job
		local carrier = EntityManager:getEntityById( self.carrierEntityId )
		if carrier then
			carrier:cancelCurrentJob()
		end
		
		Gameloop:remove( self )
		Renderer:remove( self, self.rendererLayer )
		self = nil
	end
	
	return entity
end


return ConstructionFence
