local WaterGunTower = {}
local lg = love.graphics


function WaterGunTower:create( data )
	local entity = Building:create( data )
	
	entity.entityType = ENTITIES.waterGunTower
	entity.isDefendableBuilding = true
	
	-- range in full tiles
	entity.explorationRange = 4
	
	entity.image = Images["water_gun_tower_1"]	-- need different images or animations when rotating the gun
	entity.rendererLayer = "entities"
	
	entity.state = "idle"
	
	-- selection frame image
	entity.imageWidth = 128
	entity.imageHeight = 128
	entity.outlineOffsetY = 32
	local frameImageData = Images["entity_selection_frame_1"]
	entity.selectionFrameImage = frameImageData
	entity.selectionFrameScaleX = entity.imageWidth / frameImageData.image:getWidth()
	entity.selectionFrameScaleY = entity.imageHeight / frameImageData.image:getHeight()
	
	-- blocking info in mini tiles
	entity.blocking = {
		{0,0,0,0,0},
		{0,1,1,1,0},
		{0,1,1,1,0},
		{0,1,1,1,0},
		{0,0,0,0,0},
	}
	
	----------------------------------------------------------------------------------------------------
	
	function entity:load( id )
		self.id = id
		
		Gameloop:add( self )
		Renderer:add( self, self.rendererLayer )
		
		self.ohwSelection = OhwEntitySelection:create{
			entityId = self.id,
			width = self.imageWidth,
			height = self.imageHeight,
			offsetY = self.outlineOffsetY,
		}
		
		Game:onBuildingCreated( self )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:tick( dt )
		
		-- behavior depending on the internal state
		self:behaviorTick( dt )
		
		self.timeSinceLastHealthChange = self.timeSinceLastHealthChange + dt
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:behaviorTick( dt )
		
		local ex, ey = self:getPosition()
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:switchToState( newState )
		
		self.state = newState
		-- print( "water gun tower: switch to state", newState )
		
		-- TODO: update or reset values?
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:getEnemiesInAttackRange()
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:draw()
		local body = self.body
		local x, y = convertToDimetric( body.x, body.y )
		local imgData = self.image
		local ox = imgData.ox or 0
		local oy = imgData.oy or 0
		
		lg.draw( imgData.image, x, y, 0, 1, 1, ox, oy )
		
		-- draw "light"
		-- TODO: do not draw when the building is not operating at the moment
		if true then
			local liSize = 800
			local alpha = 0.8
			local color = { 1, 0.7, 0.7, alpha }
			Renderer:drawCircularLight( x, y, liSize, color )
		end
		
		self.ohwSelection:setVisible( self.selected == true )
		
		-- healthbar
		-- should be about 4 to 6 times as wide as high and above the entity
		-- the width MIGHT be larger for entities with more health (?)
		if Game.drawEntityStatusOverhead or self.selected
		or self.timeSinceLastHealthChange < 4 then
			local hb_y_offset = body.z + 100	-- draw above the entity
			self.health:draw( x, y - hb_y_offset, 50, 8 )
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:destroy()
		Gameloop:remove( self )
		Renderer:remove( self, self.rendererLayer )
		self.ohwSelection:destroy()
		self = nil
	end
	
	
	return entity
end


return WaterGunTower
