local TeslaBolt = {}
local lg = love.graphics

-- drawing in world coordinates seems difficult...
-- add an extra renderer for projectiles maybe?
-- which uses screen coordinates; so the offset along the vertical screen axis can easily be applied
-- (projectiles do not originate from the bottom of the source entity...)

function TeslaBolt:create( data )
	local entity = {}
	
	entity.entityType = ENTITIES.teslaBolt
	entity.entityCategory = ENTITYCATEGORY.projectile
	
	entity.body = {
		shape = "point",
		x = data.x or 0,
		y = data.y or 0,
	}
	
	entity.targetId = data.targetId
	-- entity.attackDamage = data.damage or 1
	entity.ttl = data.ttl or 0.25
	
	entity.rendererLayer = "projectiles"
	
	----------------------------------------------------------------------------------------------------
	
	function entity:load( id )
		self.id = id
		
		Gameloop:add( self )
		Renderer:add( self, self.rendererLayer )
		
		-- set position of the target
		local target = EntityManager:getEntityById( self.targetId )
		if EntityManager:isValidEntity( target ) then
			
			local convertXY = convertToDimetric
			local random = math.random
			local round = Tools.round
			local points = {}
			
			local body = self.body
			local x0, y0 = convertXY( body.x, body.y )
			local target_x, target_y = convertXY( target:getPosition() )
			if target.body.z then
				target_y = target_y - target.body.z
			end
			-- print( "target:", target_x, target_y )
			
			table.insert( points, x0 )
			table.insert( points, y0 )
			
			local dx = target_x - x0
			local dy = target_y - y0
			local distance = math.sqrt( dx*dx + dy*dy )
			-- print( "distance:", distance )
			local n = random( 5, 8 )
			local drift = 20
			-- limit the random drift and number of "bends" if the target is very close
			-- for better visuals; the value is determined experimentally
			if distance < 100 then
				n = 2
				drift = 10
			end
			for i = 1, n-1 do
				local x = round( x0 + dx * i / n ) + random(-drift,drift)
				local y = round( y0 + dy * i / n ) + random(-drift,drift)
				table.insert( points, x )
				table.insert( points, y )
			end
			
			table.insert( points, target_x )
			table.insert( points, target_y )
			self.linePoints = points
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:getPosition()
		return self.body.x, self.body.y
	end
	
	
	function entity:setPosition( x, y )
		self.body.x = x
		self.body.y = y
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:tick( dt )
		
		self.ttl = self.ttl - dt
		if self.ttl <= 0 then
			EntityManager:remove( self )
			return
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:draw()
		
		if not self.linePoints then
			return
		end
		
		-- local body = self.body
		-- local x, y = convertToDimetric( body.x, body.y )
		
		for lineWidth = 4, 2, -1 do
			lg.setLineWidth( lineWidth )
			lg.setColor( 0.1, 0.5, 1, 0.2 )
			lg.line( self.linePoints )
		end
		
		lg.setLineWidth( 1 )
		lg.setColor( 1, 1, 1, 0.4 )
		lg.line( self.linePoints )
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:destroy()
		Gameloop:remove( self )
		Renderer:remove( self, self.rendererLayer )
		self = nil
	end
	
	
	return entity
end


return TeslaBolt
