local Debris = {}
local lg = love.graphics

local imageData = {}
imageData["normal"] = {}
imageData["normal"][1] = Images["debris_normal_small"]
imageData["normal"][2] = Images["debris_normal_half"]
imageData["normal"][3] = Images["debris_normal_full"]


function Debris:create( data )
	local entity = {}
	
	entity.entityType = ENTITIES.debris
	entity.entityCategory = ENTITYCATEGORY.debris
	
	entity.body = {
		x = data.x or 0,
		y = data.y or 0,
		sizeX = 125,	-- ?
		sizeY = 125,
	}
	
	-- full debris must be worked on by a RR 3x until it is completely cleared
	entity.mass = 3
	
	-- also store the tile index (avoid to calculate them over and over again)
	entity.xi = data.xi
	entity.yi = data.yi
	
	entity.rendererLayer = "ground_entities"
	entity.image = imageData["normal"][3]
	
	-- if the debris was not created after drilling a wall, there will be no ore
	-- e.g. from a landslide
	if data.createdFromLandslide then
		entity.createdFromLandslide = true
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:load( id )
		self.id = id
		
		Gameloop:add( self )
		Renderer:add( self, self.rendererLayer )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:getPosition()
		return self.body.x, self.body.y
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- set currently selected - true / false
	function entity:setSelected( selected )
		self.selected = selected
		self.animationTick = 0
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:tick( dt )
		
		if self.selected then
			self.animationTick = self.animationTick + dt
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:draw()
		local body = self.body
		local x, y = convertToDimetric( body.x, body.y )
		local imgData = self.image
		local ox = imgData.ox or 0
		local oy = imgData.oy or 0
		
		if self.selected then
			-- a simple but decent looking "animation"; value goes from 0.6 to 1.0
			local v = 0.2 * math.sin( 3 * self.animationTick ) + 0.8
			-- lg.setColor( 0.8, 0.8, v )
			lg.setColor( v, v, 1 )
		end
		-- draw it a bit smaller... (scale = 1 equals a full tile in this case)
		local scale = 0.8
		lg.draw( imgData.image, x, y, 0, scale, scale, ox, oy )
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- reduce some of the debris mass
	-- normal: 1 for a RR
	function entity:removeMass( step )
		self.mass = self.mass - (step or 1)
		if self.mass > 0 then
			self:updateImage()
		else
			self:removeLastMass()
		end
	end
	
	-- called from self:removeMass
	--> right before the debris is gone
	function entity:removeLastMass()
		Game:onBeforeDebrisRemoved( self )
		-- self:destroy()	-- NO!
		EntityManager:remove( self )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:updateImage()
		self.image = imageData["normal"][self.mass]
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:destroy()
		Gameloop:remove( self )
		Renderer:remove( self, self.rendererLayer )
		self = nil
	end
	
	
	return entity
end

return Debris
