local HardRock = {}
local lg = love.graphics

local waitBeforeDrawTransparent = 2

local imageData = {}
imageData["normal"] = {}
imageData["normal"]["default"] = Images["wall_normal_hard_rock_full"]	-- fallback option, if valid image is missing
imageData["normal"]["full"] = Images["wall_normal_hard_rock_full"]
imageData["normal"]["straight_1"] = Images["wall_normal_hard_rock_straight_1"]
imageData["normal"]["straight_2"] = Images["wall_normal_hard_rock_straight_2"]
imageData["normal"]["straight_3"] = Images["wall_normal_hard_rock_straight_3"]
imageData["normal"]["straight_4"] = Images["wall_normal_hard_rock_straight_4"]
imageData["normal"]["corner_1"] = Images["wall_normal_hard_rock_corner_1"]
imageData["normal"]["corner_2"] = Images["wall_normal_hard_rock_corner_2"]
imageData["normal"]["corner_3"] = Images["wall_normal_hard_rock_corner_3"]
imageData["normal"]["corner_4"] = Images["wall_normal_hard_rock_corner_4"]
imageData["normal"]["inner_corner_1"] = Images["wall_normal_hard_rock_inner_corner_1"]
imageData["normal"]["inner_corner_2"] = Images["wall_normal_hard_rock_inner_corner_2"]
imageData["normal"]["inner_corner_3"] = Images["wall_normal_hard_rock_inner_corner_3"]
imageData["normal"]["inner_corner_4"] = Images["wall_normal_hard_rock_inner_corner_4"]
imageData["normal"]["double_inner_corner_1"] = Images["wall_normal_hard_rock_double_inner_corner_1"]
imageData["normal"]["double_inner_corner_2"] = Images["wall_normal_hard_rock_double_inner_corner_2"]
-- add later: more themes (normal, ice?, ...? )


local mapping = {}
-- any of the alternatives can match for an image
mapping["full"] = {{1,1,1,1,1,1,1,1}}
mapping["straight_1"] = {{1,1,false,1,0,1,1,false}}
mapping["straight_2"] = {{1,1,1,1,1,false,0,false}}
mapping["straight_3"] = {{false,1,1,0,1,false,1,1}}
mapping["straight_4"] = {{false,0,false,1,1,1,1,1}}
mapping["inner_corner_1"] = {{1,1,0,1,1,1,1,1}}
mapping["inner_corner_2"] = {{1,1,1,1,1,1,1,0}}
mapping["inner_corner_3"] = {{1,1,1,1,1,0,1,1}}
mapping["inner_corner_4"] = {{0,1,1,1,1,1,1,1}}
mapping["double_inner_corner_1"] = {{1,1,0,1,1,0,1,1}}
mapping["double_inner_corner_2"] = {{0,1,1,1,1,1,1,0}}
-- 2x false?
mapping["corner_1"] = {{1,1,false,1,0,false,0,false}, {1,1,0,1,1,false,0,1}, {1,1,false,1,0,0,1,1}}
mapping["corner_2"] = {{false,1,1,0,1,false,0,false}, {false,1,1,0,1,false,1,0}, {0,1,1,1,1,1,0,false}}
mapping["corner_3"] = {{false,0,false,0,1,false,1,1}, {1,0,false,1,1,0,1,1}, {1,1,0,0,1,false,1,1}}
mapping["corner_4"] = {{false,0,false,1,0,1,1,false}, {0,1,false,1,0,1,1,false}, {false,0,1,1,1,1,1,0}}
-- mapping["corner_1"] = {{1,1,false,1,false,false,0,false}, {1,1,false,1,0,false,false,false}}
-- mapping["corner_2"] = {{false,1,1,false,1,false,0,false}, {false,1,1,0,1,false,false,false}}
-- mapping["corner_3"] = {{false,false,false,0,1,false,1,1}, {false,0,false,false,1,false,1,1}}
-- mapping["corner_4"] = {{false,false,false,1,0,1,1,false}, {false,0,false,1,false,1,1,false}}


function HardRock:create( data )
	local entity = {}
	
	entity.entityType = ENTITIES.hardRock
	entity.entityCategory = ENTITYCATEGORY.wall
	
	entity.body = {
		x = data.x or 0,
		y = data.y or 0,
		sizeX = 125,	-- ?
		sizeY = 125,
	}
	
	-- also store the tile index (avoid to calculate them over and over again)
	entity.xi = data.xi
	entity.yi = data.yi
	
	entity.drawInFogOfWar = true
	
	entity.rendererLayer = "entities"
	entity.image = imageData["normal"].default
	
	-- wall can be drilled at all
	entity.canBeDrilled = true
	
	-- it is possible that landslides occur originating from this wall
	-- also store when the last landslide occured to avoid to frequent ones
	entity.allowLandslides = true
	entity.lastLandslideTime = 0
	
	-- how long it will take to completely drill down the wall
	-- entities have an efficiency for drilling which also influences the duration
	entity.drillEndurance = 160
	
	-- 2d hitbox for mouse interaction
	entity.imageWidth = entity.image.image:getWidth()
	entity.imageHeight = entity.image.image:getHeight()
	entity.outlineOffsetY = 50
	
	entity.drawTransparentTimer = 0
	
	----------------------------------------------------------------------------------------------------
	
	function entity:load( id )
		self.id = id
		
		Gameloop:add( self )
		Renderer:add( self, self.rendererLayer )
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- renamed from get2dHitbox to avoid this entity is selected by game:getEntityAtScreenPosition
	function entity:_get2dHitbox()
		local x, y = convertToDimetric( self.body.x, self.body.y )
		local x, y = Camera:getTransform():transformPoint( x, y )
		local cameraScale = Camera:getScale()
		return {
			shape = "rect",
			x = Tools.round(x),
			-- do not forget the camera scale here!
			y = Tools.round(y) - self.outlineOffsetY / cameraScale,
			sizeX = self.imageWidth / cameraScale,
			sizeY = self.imageHeight / cameraScale,
		}
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:getPosition()
		return self.body.x, self.body.y
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- set currently selected - true / false
	function entity:setSelected( selected )
		self.selected = selected
		
		self.animationTick = 0
		-- if selected then
		-- else
		-- end
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- called from game, when a unit is drilling this wall
	function entity:setRemainingDrillEndurance( endurance )
		self.drillEndurance = endurance
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- called when a landslide occurs from this wall
	function entity:setLastLandslideTime( gametime )
		self.lastLandslideTime = gametime
	end
	
	
	function entity:getLastLandslideTime()
		return self.lastLandslideTime
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:tick( dt )
		
		if self.selected then
			self.animationTick = self.animationTick + dt
		end
		
		-- to show objects behind the wall...
		-- make the wall semi-transparent while the mouse is over the upper part of the wall image
		--[[
		do
			local mousePosScreen = Mouse:getPosition()	-- mouse pos on screen
			local wx, wy = convertTo2D( Camera:getTransform():inverseTransformPoint( mousePosScreen.x, mousePosScreen.y ) )
			local xi, yi = getTileIndex( wx, wy )
			local mousePosWorld = { x = wx, y = wy }		-- mouse pos in the world
			if Tools.isPointInsideShape( mousePosScreen, self:_get2dHitbox() )
			and (xi < self.xi or yi < self.yi) then
				self.drawTransparentTimer = self.drawTransparentTimer + dt
			else
				self.drawTransparentTimer = 0
			end
		end
		--]]
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- called from game
	-- change the image depending on the 8 neighbours (wall or not)
	-- code is a table with 8 entries of 0 (no wall) or 1 (wall) representing:
	-- 123
	-- 4_5
	-- 678
	function entity:updateImageCode( code )
		
		for src, conditions in pairs( mapping ) do
			for _, condition in pairs( conditions ) do
				local comply = true
				for i, v in ipairs( condition ) do
					if v and code[i] ~= v then
						comply = false
						break
					end
				end
				if comply then
					self.image = imageData["normal"][src]
					return true
				end
			end
		end
		
		print( "wall:updateImageCode - no match found, use default" )
		self.image = imageData["normal"]["default"]
		return false
	end
	
	
	-- function entity:updateImageCode( code )
		-- local str = table.concat( code )	-- string representation of the code
		-- local source = "default"
		
		-- if code[1] == 1 and code[2] == 1 and code[3] == 1 and code[4] == 1
		-- and code[5] == 1 and code[6] == 1 and code[7] == 1 and code[8] == 1 then
			-- source = "full"
			
		-- elseif code[1] == 1 and code[2] == 1 and code[4] == 1
		-- and code[5] == 0 and code[6] == 1 and code[7] == 1 then
			-- source = "straight_1"
			
		-- elseif code[1] == 1 and code[2] == 1 and code[3] == 1
		-- and code[4] == 1 and code[5] == 1 and code[7] == 0 then
			-- source = "straight_2"
			
		-- elseif str == "11111110" then
			-- source = "inner_corner_2"
		-- end
		
		-- self.image = imageData["normal"][source]
	-- end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:draw()
		local body = self.body
		local x, y = convertToDimetric( body.x, body.y )
		local imgData = self.image
		local ox = imgData.ox or 0
		local oy = imgData.oy or 0
		
		-- to show objects behind the wall...
		-- make the wall semi-transparent while the mouse is over the upper part of the wall image
		-- but because this might be annoying when not intended, only activate when a key is held down
		local drawTransparent = false
		if love.keyboard.isDown( Keybinds:getKey("transparent_walls") ) then
			local mousePosScreen = Mouse:getPosition()	-- mouse pos on screen
			local wx, wy = convertTo2D( Camera:getTransform():inverseTransformPoint( mousePosScreen.x, mousePosScreen.y ) )
			local xi, yi = getTileIndex( wx, wy )
			local mousePosWorld = { x = wx, y = wy }		-- mouse pos in the world
			local fow = Game.fow
			if Tools.isPointInsideShape( mousePosScreen, self:_get2dHitbox() )
			and fow:isTileDiscovered( xi, yi )
			and (xi < self.xi or yi < self.yi) then
				drawTransparent = true
			end
		end
		
		-- colored outline represents that this wall is reinforced
		if self.isReinforced and not drawTransparent then
			lg.setColor( COLORS.yellow )
			lg.draw( imgData.image, x, y, 0, 1.01, 1.01, ox, oy )
			lg.setColor( COLORS.white )
		end
		
		if self.selected then
			-- a simple but decent looking "animation"; value goes from 0.6 to 1.0
			local v = 0.2 * math.sin( 3 * self.animationTick ) + 0.8
			-- lg.setColor( 0.8, 0.8, v )
			lg.setColor( v, v, 1 )
		end
		
		if drawTransparent then
			local r,g,b = lg.getColor()
			lg.setColor( r, g, b, 0.5 )
		end
		lg.draw( imgData.image, x, y, 0, 1, 1, ox, oy )
		
		-- lg.setColor( 1, 1, 1 )
		-- lg.circle( "fill", x, y, 6 )	-- debug
		
		-- 2d hitbox (mouse interaction)
		-- lg.setColor( 1, 1, 1, 0.2 )
		-- lg.rectangle( "line", x - self.imageWidth * 0.5, y - self.imageHeight * 0.5 - self.outlineOffsetY, self.imageWidth, self.imageHeight )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:destroy()
		Gameloop:remove( self )
		Renderer:remove( self, self.rendererLayer )
		self = nil
	end
	
	
	return entity
end

return HardRock
