local Landslide = {}
local lg = love.graphics

-- how long will the landslide be alive
local ttl = 2
local damageIntervall = 0.2	-- deal damage every X seconds


function Landslide:create( data )
	local entity = {}
	
	entity.entityType = ENTITIES.landslide
	entity.entityCategory = ENTITYCATEGORY.landslide
	
	entity.body = {
		x = data.x or 0,
		y = data.y or 0,
	}
	entity.xi = data.xi
	entity.yi = data.yi
	
	entity.rendererLayer = "entities"
	
	entity.ttl = data.ttl or ttl
	
	entity.damageTick = 0
	
	----------------------------------------------------------------------------------------------------
	
	function entity:load( id )
		self.id = id
		
		Gameloop:add( self )
		Renderer:add( self, self.rendererLayer )
		
		local img = Images["ore_gray"].image
		self.particleSystem = lg.newParticleSystem( img, 16 )
		-- self.particleSystem:setEmitterLifetime( ttl )
		self.particleSystem:setParticleLifetime( 1.6, 2 )
		self.particleSystem:setSizes( 0.5, 1, 1, 0.5 )
		-- self.particleSystem:setSizeVariation( 1 )
		self.particleSystem:setEmissionArea( "uniform", 96, 32, 0 )
		self.particleSystem:setLinearAcceleration( -8, 32, 8, 64 )
		self.particleSystem:setSpin( 0, 2 )	-- in rad per second
		self.particleSystem:emit( 16 )
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:getPosition()
		return self.body.x, self.body.y
	end
	
	
	function entity:setPosition( x, y )
		self.body.x = x
		self.body.y = y
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:tick( dt )
		self.particleSystem:update( dt )
		
		self.ttl = self.ttl - dt
		if self.ttl <= 0 then
			EntityManager:remove( self )
			return
		end
		
		-- deal damage to units at this tile every (damageIntervall) seconds
		self.damageTick = self.damageTick + dt
		if self.damageTick >= damageIntervall then
			self.damageTick = self.damageTick - damageIntervall
			
			local xi, yi = self.xi, self.yi
			local damagePerSecond = 50	-- TODO
			local damage = damagePerSecond * damageIntervall
			local entities = EntityManager:getEntitiesFilter( function(e)
				if not e.health then return false end
				-- rock raiders inside vehicles do not take damage (but the vehicle does)
				if e.entityType == ENTITIES.rockRaider and e.vehicleEntity then return false end
				local exi, eyi
				if e.xi and e.yi then
					exi = e.xi
					eyi = e.yi
				else
					local x, y = e:getPosition()
					exi, eyi = getTileIndex( x, y )
				end
				return exi == xi and eyi == yi
				-- can take damage from landslide condition? TODO
				-- flying mobs do not take damage?
			end )
			for i, entity in pairs( entities ) do
				Game:onEntityHurtEntity( self, entity, damage )
			end
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:draw()
		
		local body = self.body
		local x, y = convertToDimetric( body.x, body.y )
		lg.draw( self.particleSystem, x, y - 128 )
		
		-- lg.circle( "fill", x, y, 8 )	-- debug
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:destroy()
		Gameloop:remove( self )
		Renderer:remove( self, self.rendererLayer )
		self = nil
	end
	
	
	return entity
end


return Landslide
