local FogOfWar = {}

-- once value per tile:

-- *option 1:
-- 0 = not discovered yet
-- 1 = discovered, but currently not visible
-- 2 = discovered and also currently explored
--> update all entity exploration oncer per second

-- option 2:
-- -1 = not discovered yet
-- 0 = discovered, but currently not visible
-- 1+ = discovered and also currently explored by X units
--> every unit adds 1 to the exploration value (easy updating?)

function FogOfWar:create( width, height )
	local fow = { width = width, height = height, exploration = {} }
	
	-- prepare the blocking info: all empty
	for y = 1, fow.height do
		fow.exploration[y] = {}
		for x = 1, fow.width do
			fow.exploration[y][x] = 0
		end
	end
	
	
	function fow:isValidTileIndex( xi, yi )
		if xi < 1 or xi > self.width
		or yi < 1 or yi > self.height then
			return false
		end
		return true
	end
	
	
	-- check if the tile is CURRENTLY visible to the player
	function fow:isTileExplored( xi, yi )
		if not self:isValidTileIndex( xi, yi ) then
			return false
		end
		return self.exploration[yi][xi] == 2
	end
	
	
	-- check if the tile has been discovered already (it is currently or at least was explored once)
	function fow:isTileDiscovered( xi, yi )
		if not self:isValidTileIndex( xi, yi ) then
			return false
		end
		return self.exploration[yi][xi] > 0
	end
	
	
	-- explored right now
	-- return true if the tile was discovered now for the first time
	function fow:setTileExplored( xi, yi )
		if not self:isValidTileIndex( xi, yi ) then
			return false
		end
		local notDiscoveredBefore = self.exploration[yi][xi] == 0
		self.exploration[yi][xi] = 2
		return notDiscoveredBefore
	end
	
	
	function fow:prepareUpdateTick()
		for yi = 1, self.height do
			for xi = 1, self.width do
				self.exploration[yi][xi] = self.exploration[yi][xi] == 0 and 0 or 1
			end
		end
	end
	
	
	return fow
end

return FogOfWar
