local Health = {}
local lg = love.graphics

-- health component for entities that have health / can take damage

function Health:create( entity, maxHealth, canRegenerate, regenerationAmount, currentHealth )
	
	local self = {
		entity = entity,
		maxHealth = maxHealth,
		canRegenerate = canRegenerate or false,
		regenerationAmount = regenerationAmount or 0,
		health = currentHealth or maxHealth,
	}
	
	entity.timeSinceLastHealthChange = entity.timeSinceLastHealthChange or 0
	
	----------------------------------------------------------------------------------------------------
	
	function self:getMaxHealth()
		return self.maxHealth
	end
	
	
	function self:setMaxHealth( health )
		self.maxHealth = health
		-- self:onHealthChanged()	-- ?
		return self.maxHealth
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- absolute value
	function self:getHealth()
		return self.health
	end
	
	
	function self:setHealth( health )
		self.health = Tools.clamp( health, 0, self:getMaxHealth() )
		self:onHealthChanged()
		return self.health
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- % value
	function self:getHealthPercent()
		return 100 * self:getHealth() / self:getMaxHealth()
	end
	
	
	function self:setHealthPercent( hp )
		local hp = Tools.clamp( hp, 0, 100 )
		local newHealth = self:setHealth( self:getMaxHealth() * hp * 0.01 )
		return newHealth
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- value between 0 and 1
	function self:getHealthRatio()
		return self:getHealth() / self:getMaxHealth()
	end
	
	
	function self:setHealthRatio( ratio )
		local ratio = Tools.clamp( ratio, 0, 1 )
		local newHealth = self:setHealth( self:getMaxHealth() * ratio )
		return newHealth
	end
	
	----------------------------------------------------------------------------------------------------
	
	function self:hurt( _damage )
		local health = self:getHealth()
		local damage = Tools.min( _damage, health )	-- cannot take more damage then current health
		local newHealth = self:setHealth( health - damage )
		if self.entity and self.entity.onHurt then
			self.entity:onHurt( damage )
		end
		return newHealth
	end
	
	----------------------------------------------------------------------------------------------------
	
	function self:onHealthChanged()
		if self.entity and self.entity.onHealthChanged then
			self.entity:onHealthChanged()
		else
			print( "health - entity has no health component!" )
		end
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- called from the associated entity with the screen coordinates where to draw the health bar
	function self:draw( x, y, w, h, color )
		local healthRatio = self:getHealthRatio()
		local hb_width = w or 32
		local hb_height = h or 6
		local hb_x = x - hb_width * 0.5
		local hb_y = y - hb_height * 0.5
		lg.setColor( COLORS.hb_black )
		lg.rectangle( "fill", hb_x, hb_y, hb_width, hb_height )
		local barColor = color or COLORS.hb_green
		lg.setColor( barColor )
		lg.rectangle( "fill", hb_x, hb_y, hb_width * healthRatio, hb_height )
		lg.setColor( COLORS.white )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function self:remove()
		self = nil
	end
	
	----------------------------------------------------------------------------------------------------
	
	return self
end


return Health
