-- GUI elements for the game screen
-- GUI elements for the game screen
local GameScreen = {}


function GameScreen:create()
	local game = {}
	
	game.id = "game"
	
	function game:open()
		GUImgr:setScreen( self.id )
		
		Music:switchSetToUse( "game" )
		
		Game:open()
	end
	
	----------------------------------------------------------------------------------------------------
	
	function game:switchToSelectionState( newState )
		
		-- optional: perform something to clean up the old state?
		GUImgr:setVisibility( "widGameGlobalSelection", false )
		GUImgr:setVisibility( "widGameWallSelection", false )
		GUImgr:setVisibility( "widGameBuildingSelection", false )
		GUImgr:setVisibility( "widGameEntitySelection", false )
		GUImgr:setVisibility( "widGameTileSelection", false )
		
		-- and show and hide the correct widgets and buttons...
		if newState == "global" then
			GUImgr:setVisibility( "widGameGlobalSelection", true )
			
		elseif newState == "wall" then
			GUImgr:setVisibility( "widGameWallSelection", true )
			
		elseif newState == "building" then
			GUImgr:setVisibility( "widGameBuildingSelection", true )
			
		elseif newState == "entity" then
			GUImgr:setVisibility( "widGameEntitySelection", true )
			
		elseif newState == "tile" then
			GUImgr:setVisibility( "widGameTileSelection", true )
			
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- GLOBAL SELECTION STATE = NO ENTITY IS CURRENTLY SELECTED
	Widget:create{
		id = "widGameGlobalSelection",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.5,
		screenId = "game",
		visible = false,
	}
	
	-- WALL SELECTION
	Widget:create{
		id = "widGameWallSelection",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.5,
		screenId = "game",
		visible = false,
	}
	
	-- BUILDING SELECTION
	Widget:create{
		id = "widGameBuildingSelection",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.5,
		screenId = "game",
		visible = false,
	}
	
	-- ENTITY SELECTION
	Widget:create{
		id = "widGameEntitySelection",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.5,
		screenId = "game",
		visible = false,
	}
	
	-- (EMPTY OR WITH DEBRIS?) TILE SELECTION
	Widget:create{
		id = "widGameTileSelection",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.5,
		screenId = "game",
		visible = false,
	}
	
	-- BUILDIG MENU (select new building to create at selected tile)
	Widget:create{
		id = "widGameBuildingMenu",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.8,
		screenId = "game",
		visible = false,
	}
	
	-- NEW VEHICLE MENU (opened from vehicle factory)
	Widget:create{
		id = "widGameVehicleFactoryNewVehicleMenu",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.3,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.4,
		screenId = "game",
		visible = false,
	}
	
	-- NEW ROBOT MENU (opened from robot bay)
	Widget:create{
		id = "widGameRobotBayNewRobotMenu",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.15,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		screenId = "game",
		visible = false,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- SELECTED WALL
	
	-- MINE IT
	Button:create{
		id = "btnGameActionDestroyWall",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_drill_wall"],
		callback = function()
			Game:guiActionDrillWall()
		end,
		screenId = "game",
		parentId = "widGameWallSelection",
		tooltip = { text = getString( "game_action_drill_wall_tooltip" ) },
	}
	
	-- REINFORCE IT
	Button:create{
		id = "btnGameActionReinforceWall",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.15,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_reinforce_wall"],
		callback = function()
			Game:guiActionReinforceWall()
		end,
		screenId = "game",
		parentId = "widGameWallSelection",
		tooltip = { text = getString( "game_action_reinforce_wall_tooltip" ) },
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- SELECTED TILE
	
	-- REMOVE DEBRIS
	Button:create{
		id = "btnGameActionRemoveDebris",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_clear_debris"],
		callback = function()
			
			-- debris (tile) selected? -> add a task to a job queue
			local debris = Game.selectedEntity
			if not debris
			or debris.entityType ~= ENTITIES.debris then
				return false
			end
			
			-- DEBUG
			if love.keyboard.isDown( "lshift" ) then
				-- game:debugDestroySelectedDebris()
				debris:removeMass()
				-- game:reselectCurrentTile()
				return
			end
			-- /DEBUG
			
			Game:addClearDebrisJob( debris )
			
		end,
		screenId = "game",
		parentId = "widGameTileSelection",
		tooltip = { text = getString( "game_action_remove_debris_tooltip" ) },
	}
	
	-- CREATE BUILDING AREA - deprecated?
	--[[
	Button:create{
		id = "btnGameActionCreateBuildingArea",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.15,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		text = "Bauplatte errichten",
		autoSizeSquare = true,
		-- fSize = 22,
		callback = function()
			-- Game:debugCreateBuildingArea()
		end,
		screenId = "game",
		parentId = "widGameTileSelection",
		enabled = true,
	}
	--]]
	
	-- OPEN BUILDING MENU
	Button:create{
		id = "btnGameToggleBuildingMenu",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = "Gebäude bauen",
		imageData = Images["icon_construct_building"],
		-- fSize = 22,
		callback = function()
			if Game.isPaused then return end
			GUImgr:toggleVisibility( "widGameBuildingMenu" )
			-- hide conflicting widgets
			-- GUImgr:setVisibility( "widGameWorkerJobPriorities", false )
		end,
		screenId = "game",
		parentId = "widGameTileSelection",
		tooltip = { text = getString( "game_building_menu_tooltip" ) },
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- building buttons
	do
		
		-- see position and size of widGameBuildingMenu!
		local buttonSize = Tools.min( SCREEN.width * 0.09, SCREEN.height * 0.09 )
		local x = SCREEN.width * 0.85
		local top = SCREEN.height * 0.15
		local buildingButtons = {
			{ id = "btnGameActionBuildToolStore", icon = Images["icon_tool_store"],
				entityType = ENTITIES.toolStore, tooltipKey = "game_build_tool_store_tooltip" },
			{ id = "btnGameActionBuildTeleportPad", icon = Images["icon_teleport_pad"],
				entityType = ENTITIES.teleportPad, tooltipKey = "game_build_teleport_pad_tooltip" },
			-- { id = "btnGameActionBuildResearchCenter", icon = Images["icon_construct_building"], entityType = nil, },	-- TODO
			{ id = "btnGameActionBuildVehicleFactory", icon = Images["icon_vehicle_factory"],
				entityType = ENTITIES.vehicleFactory, tooltipKey = "game_build_vehicle_factory_tooltip" },
			{ id = "btnGameActionBuildRobotBay", icon = Images["icon_robot_bay"],
				entityType = ENTITIES.robotBay, tooltipKey = "game_build_robot_bay_tooltip" },
			{ id = "btnGameActionBuildTeslaTower", icon = Images["icon_tesla_tower"],
				entityType = ENTITIES.teslaTower, tooltipKey = "game_build_tesla_tower_tooltip" },
			{ id = "btnGameActionBuildWaterGunTower", icon = Images["icon_water_gun_tower"],
				entityType = ENTITIES.waterGunTower, tooltipKey = "game_build_water_gun_tower_tooltip" },
			{ id = "btnGameActionBuildSupportStation", icon = Images["icon_support_station"],
				entityType = ENTITIES.supportStation, tooltipKey = "game_build_support_station_tooltip" },
		}
		for i, btn in ipairs( buildingButtons ) do
			local x = x
			local y = top + (i-1) * SCREEN.height * 0.1
			Button:create{
				id = btn.id,
				x = x,
				y = y,
				sizeX = buttonSize,
				sizeY = buttonSize,
				imageData = btn.icon,
				callback = function()
					Game:guiActionCreateBuilding( btn.entityType )
				end,
				screenId = "game",
				parentId = "widGameBuildingMenu",
				tooltip = {
					text = getString( btn.tooltipKey ),
				},
			}
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- VEHICLE FACTORY: TOGGLE NEW VEHICLE MENU
	Button:create{
		id = "btnGameVehicleFactoryToggleNewVehicleMenu",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = "Fahrzeuge",
		-- fSize = 22,
		imageData = Images["icon_vehicle"],
		callback = function()
			-- GUImgr:setVisibility( "widGameWorkerJobPriorities", false )
			GUImgr:toggleVisibility( "widGameVehicleFactoryNewVehicleMenu" )
		end,
		screenId = "game",
		parentId = "widGameBuildingSelection",
		enabled = true,
	}
	
	-- new vehicle buttons
	do
		local buttonSize = Tools.min( SCREEN.width * 0.09, SCREEN.height * 0.09 )
		local x = SCREEN.width * 0.85
		local top = SCREEN.height * 0.15
		local buttons = {
			{ id = "btnGameActionCreateHoverScout", icon = Images["icon_hover_scout"], entityType = ENTITIES.hoverScout, },
			{ id = "btnGameActionCreateDigger", icon = Images["icon_digger"], entityType = ENTITIES.digger, },
			{ id = "btnGameActionCreateTransportTruck", icon = Images["icon_transport_truck"], entityType = ENTITIES.transportTruck, },
			{ id = "btnGameActionCreateMobileLaserCutter", icon = Images["icon_mobile_laser_cutter"], entityType = ENTITIES.mobileLaserCutter, },
		}
		
		for i, btn in ipairs( buttons ) do
			Button:create{
				id = btn.id,
				x = x,
				y = top + (i-1) * SCREEN.height * 0.1,
				shape = "rect",
				sizeX = buttonSize,
				sizeY = buttonSize,
				imageData = btn.icon,
				callback = function()
					Game:guiActionCreateVehicle( btn.entityType )
				end,
				screenId = "game",
				parentId = "widGameVehicleFactoryNewVehicleMenu",
			}
		end
	end
	
	--[[
	-- NEW VEHICLE: HOVER SCOUT
	Button:create{
		id = "btnGameActionCreateHoverScout",
		x = SCREEN.width * 0.6,
		y = SCREEN.height * 0.2,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = "Hover Scout",
		-- fSize = 22,
		imageData = Images["icon_hover_scout"],
		callback = function()
			Game:guiActionCreateVehicle( ENTITIES.hoverScout )
		end,
		screenId = "game",
		parentId = "widGameVehicleFactoryNewVehicleMenu",
		enabled = true,
	}
	
	-- NEW VEHICLE: DIGGER
	Button:create{
		id = "btnGameActionCreateDigger",
		x = SCREEN.width * 0.7,
		y = SCREEN.height * 0.2,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = "Buddler",
		-- fSize = 22,
		imageData = Images["icon_digger"],
		callback = function()
			Game:guiActionCreateVehicle( ENTITIES.digger )
		end,
		screenId = "game",
		parentId = "widGameVehicleFactoryNewVehicleMenu",
		enabled = true,
	}
	
	-- NEW VEHICLE: TRANSPORT TRUCK
	Button:create{
		id = "btnGameActionCreateTransportTruck",
		x = SCREEN.width * 0.8,
		y = SCREEN.height * 0.2,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = "Transporter",
		-- fSize = 22,
		imageData = Images["icon_transport_truck"],
		callback = function()
			Game:guiActionCreateVehicle( ENTITIES.transportTruck )
		end,
		screenId = "game",
		parentId = "widGameVehicleFactoryNewVehicleMenu",
		enabled = true,
	}
	
	-- NEW VEHICLE: MOBILE LASER CUTTER
	Button:create{
		id = "btnGameActionCreateMobileLaserCutter",
		x = SCREEN.width * 0.6,
		y = SCREEN.height * 0.3,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_mobile_laser_cutter"],
		callback = function()
			Game:guiActionCreateVehicle( ENTITIES.mobileLaserCutter )
		end,
		screenId = "game",
		parentId = "widGameVehicleFactoryNewVehicleMenu",
		enabled = true,
	}
	--]]
	
	----------------------------------------------------------------------------------------------------
	
	-- ROBOT BAY: TOGGLE NEW ROBOT MENU
	Button:create{
		id = "btnGameRobotBayToggleNewRobotMenu",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_probe_droid"],	-- TODO
		callback = function()
			-- GUImgr:setVisibility( "widGameWorkerJobPriorities", false )
			GUImgr:toggleVisibility( "widGameRobotBayNewRobotMenu" )
		end,
		screenId = "game",
		parentId = "widGameBuildingSelection",
	}
	
	-- NEW ROBOT: PROBE DROID
	Button:create{
		id = "btnGameActionCreateProbeDroid",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.15,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_probe_droid"],
		callback = function()
			Game:guiActionCreateRobot( ENTITIES.probeDroid )
		end,
		screenId = "game",
		parentId = "widGameRobotBayNewRobotMenu",
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- TELEPORT PAD: TELEPORT RR DOWN
	Button:create{
		id = "btnGameActionTeleportRockRaiderToBuilding",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.05,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = "Rock Raider hierher teleportieren",
		-- fSize = 22,
		callback = function()
			Game:guiActionTeleportRockRaiderToBuilding()
		end,
		screenId = "game",
		parentId = "widGameBuildingSelection",
		imageData = Images["icon_rock_raider_1"],
		enabled = true,
		tooltip = {
			text = getString( "game_teleport_rock_raider_to_building_tooltip" ),
			-- enabled = true,
			-- duration = -1,
		},
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- ROCK RAIDER, VEHICLE ETC.: EXPELL/TELEPORT UP
	Button:create{
		id = "btnGameActionExpellUnit",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.45,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = getString( "game_expell_unit" ),
		-- fSize = 22,
		imageData = Images["icon_expell_unit"],
		callback = function()
			Game:guiActionExpellUnit()
		end,
		screenId = "game",
		parentId = "widGameEntitySelection",
		visible = false,
		tooltip = {
			text = getString( "game_action_expell_unit_tooltip" ),
		},
	}
	
	-- ROCK RAIDER: TOGGLE AUTOMATIC JOB ASSIGNMENT
	Button:create{
		id = "btnGameActionRockRaiderToggleAutomaticJobAssignment",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = getString( "game_expell_unit" ),
		-- fSize = 22,
		imageData = Images["icon_settings"],	-- TODO
		callback = function()
			Game:guiActionRockRaiderToggleAutomaticJobAssignment()
		end,
		screenId = "game",
		parentId = "widGameEntitySelection",
		tooltip = {
			text = getString( "game_toggle_rock_raider_automatic_job_assignment_tooltip" ),
		},
		visible = false,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- DESTROY SELECTED BUILDING WIDGET
	Widget:create{
		id = "widGameConfirmDestroyBuilding",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.2,
		screenId = "game",
		visible = false,
	}
	
	-- OPEN DESTROY SELECTED BUILDING CONFIRMATION DIALOG
	Button:create{
		id = "btnGameOpenDestroyBuildingWidget",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.45,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = getString( "game_destroy_building" ),
		-- fSize = 22,
		imageData = Images["icon_destroy_building"],
		callback = function()
			GUImgr:setVisibility( "gameMinimap", false )
			GUImgr:setVisibility( "widGameConfirmDestroyBuilding", true )
		end,
		screenId = "game",
		parentId = "widGameBuildingSelection",
		enabled = true,
	}
	
	-- DESTROY SELECTED BUILDING CONFIRMATION TEXT
	Label:create{
		id = "lblGameConfirmDestroyBuilding",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.45,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		text = getString( "game_confirm_destroy_building" ),
		fSize = 32,
		screenId = "game",
		parentId = "widGameConfirmDestroyBuilding",
	}
	
	-- DESTROY SELECTED BUILDING
	Button:create{
		id = "btnGameActionDestroyBuilding",
		x = SCREEN.width * 0.45,
		y = SCREEN.height * 0.55,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_yes"],
		callback = function()
			Game:guiActionDestroyBuilding()
			GUImgr:setVisibility( "widGameConfirmDestroyBuilding", false )
		end,
		screenId = "game",
		parentId = "widGameConfirmDestroyBuilding",
		enabled = true,
	}
	
	-- DO NOT DESTROY SELECTED BUILDING
	Button:create{
		id = "btnGameCancelDestroyBuilding",
		x = SCREEN.width * 0.55,
		y = SCREEN.height * 0.55,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_no"],
		callback = function()
			GUImgr:setVisibility( "widGameConfirmDestroyBuilding", false )
		end,
		screenId = "game",
		parentId = "widGameConfirmDestroyBuilding",
		enabled = true,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- VEHICLE: GET DRIVER
	Button:create{
		id = "btnGameActionVehicleGetDriver",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_vehicle_get_driver"],
		callback = function()
			Game:guiActionVehicleGetDriver()
		end,
		screenId = "game",
		parentId = "widGameEntitySelection",
		visible = false,
		tooltip = { text = getString( "game_vehicle_get_driver_tooltip" ) },
	}
	
	-- VEHICLE: DRIVER LEAVE
	Button:create{
		id = "btnGameActionVehicleDriverLeave",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.15,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = getString( "game_vehicle_driver_leave" ),
		imageData = Images["icon_vehicle_driver_leave"],
		callback = function()
			Game:guiActionVehicleDriverLeave()
		end,
		screenId = "game",
		parentId = "widGameEntitySelection",
		visible = false,
		tooltip = { text = getString( "game_vehicle_driver_leave_tooltip" ) },
	}
	
	----------------------------------------------------------------------------------------------------
	
	Widget:create{
		id = "widGameResourceOverview",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.width * 0.4,
		sizeY = SCREEN.height * 0.1,
		screenId = "game",
		visible = true,
	}
	
	Button:create{
		id = "btnGameResourceAmountEnergyCrystal",
		x = SCREEN.width * 0.35,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.height * 0.08,
		sizeY = SCREEN.height * 0.08,
		text = "0",
		fSize = 22,
		imageData = Images["energy_crystal"],
		screenId = "game",
		parentId = "widGameResourceOverview",
		tooltip = { text = getString( "game_stored_energy_crystal_tooltip" ) },
	}
	
	Button:create{
		id = "btnGameResourceAmountOre",
		x = SCREEN.width * 0.45,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.height * 0.08,
		sizeY = SCREEN.height * 0.08,
		text = "0",
		fSize = 22,
		imageData = Images["ore_normal"],
		screenId = "game",
		parentId = "widGameResourceOverview",
		tooltip = { text = getString( "game_stored_ore_tooltip" ) },
	}
	
	Button:create{
		id = "btnGameResourceAmountRockSample",
		x = SCREEN.width * 0.55,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.height * 0.08,
		sizeY = SCREEN.height * 0.08,
		text = "0",
		fSize = 22,
		imageData = Images["rock_sample_1"],
		screenId = "game",
		parentId = "widGameResourceOverview",
		tooltip = { text = getString( "game_stored_rock_sample_tooltip" ) },
	}
	
	----------------------------------------------------------------------------------------------------
	
	Button:create{
		id = "btnGameActionJumpToRockRaider",
		x = SCREEN.width * 0.75,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_rock_raider_frontal"],
		callback = function()
			Game:guiActionJumpToRockRaider()
		end,
		tooltip = {
			text = getString( "game_jump_to_rock_raider_tooltip" ),
		},
		screenId = "game",
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- PRIORITIES (change the order of worker job priorities)
	Button:create{
		id = "btnGameTogglePriorityMenu",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = getString( "game_priorities" ),
		-- fSize = 22,
		imageData = Images["icon_swap_priorities"],	-- TODO
		callback = function()
			if Game.isPaused then return end
			local isVisible = GUImgr:isVisible( "widGameWorkerJobPriorities" )
			-- hide all other widgets first!
			Game:clearEntitySelection()
			-- clearEntitySelection closes this widget, so this toggle will not work
			-- GUImgr:toggleVisibility( "widGameWorkerJobPriorities" )
			GUImgr:setVisibility( "widGameWorkerJobPriorities", not isVisible )
		end,
		tooltip = {
			text = getString( "game_swap_priority_menu_tooltip" ),
		},
		screenId = "game",
	}
	
	Widget:create{
		id = "widGameWorkerJobPriorities",
		x = SCREEN.width * 0.8,
		y = SCREEN.height * 0.45,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.7,
		screenId = "game",
		visible = false,
	}
	
	-- 1st priority
	Button:create{
		id = "btnGameJobPriority1",
		x = SCREEN.width * 0.75,
		y = SCREEN.height * 0.15,
		shape = "rect",
		sizeX = SCREEN.width * 0.08,
		sizeY = SCREEN.height * 0.08,
		autoSizeSquare = true,
		text = "",
		fSize = 22,
		screenId = "game",
		parentId = "widGameWorkerJobPriorities",
		enabled = true,
	}
	
	-- 2nd priority
	Button:create{
		id = "btnGameJobPriority2",
		x = SCREEN.width * 0.75,
		y = SCREEN.height * 0.25,
		shape = "rect",
		sizeX = SCREEN.width * 0.08,
		sizeY = SCREEN.height * 0.08,
		autoSizeSquare = true,
		text = "",
		fSize = 22,
		screenId = "game",
		parentId = "widGameWorkerJobPriorities",
		enabled = true,
	}
	
	-- 3rd priority
	Button:create{
		id = "btnGameJobPriority3",
		x = SCREEN.width * 0.75,
		y = SCREEN.height * 0.35,
		shape = "rect",
		sizeX = SCREEN.width * 0.08,
		sizeY = SCREEN.height * 0.08,
		autoSizeSquare = true,
		text = "",
		fSize = 22,
		screenId = "game",
		parentId = "widGameWorkerJobPriorities",
		enabled = true,
	}
	
	-- 4th priority
	Button:create{
		id = "btnGameJobPriority4",
		x = SCREEN.width * 0.75,
		y = SCREEN.height * 0.45,
		shape = "rect",
		sizeX = SCREEN.width * 0.08,
		sizeY = SCREEN.height * 0.08,
		autoSizeSquare = true,
		text = "",
		fSize = 22,
		screenId = "game",
		parentId = "widGameWorkerJobPriorities",
		enabled = true,
	}
	
	-- 5th priority
	Button:create{
		id = "btnGameJobPriority5",
		x = SCREEN.width * 0.75,
		y = SCREEN.height * 0.55,
		shape = "rect",
		sizeX = SCREEN.width * 0.08,
		sizeY = SCREEN.height * 0.08,
		autoSizeSquare = true,
		text = "",
		fSize = 22,
		screenId = "game",
		parentId = "widGameWorkerJobPriorities",
		enabled = true,
	}
	
	-- 6th priority
	Button:create{
		id = "btnGameJobPriority6",
		x = SCREEN.width * 0.75,
		y = SCREEN.height * 0.65,
		shape = "rect",
		sizeX = SCREEN.width * 0.08,
		sizeY = SCREEN.height * 0.08,
		autoSizeSquare = true,
		text = "",
		fSize = 22,
		screenId = "game",
		parentId = "widGameWorkerJobPriorities",
		enabled = true,
	}
	
	-- 7th priority
	Button:create{
		id = "btnGameJobPriority7",
		x = SCREEN.width * 0.75,
		y = SCREEN.height * 0.75,
		shape = "rect",
		sizeX = SCREEN.width * 0.08,
		sizeY = SCREEN.height * 0.08,
		autoSizeSquare = true,
		text = "",
		fSize = 22,
		screenId = "game",
		parentId = "widGameWorkerJobPriorities",
		enabled = true,
	}
	
	-- switch 1st and 2nd
	Button:create{
		id = "btnGameSwapWorkerJobPriorities1",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.2,
		shape = "rect",
		sizeX = SCREEN.width * 0.05,
		sizeY = SCREEN.height * 0.05,
		-- text = "",
		-- fSize = 22,
		imageData = Images["icon_swap_priorities"],
		callback = function()
			Game:swapWorkerJobPriorities( 1, 2 )
		end,
		screenId = "game",
		parentId = "widGameWorkerJobPriorities",
		enabled = true,
	}
	
	-- switch 2nd and 3rd
	Button:create{
		id = "btnGameSwapWorkerJobPriorities2",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.3,
		shape = "rect",
		sizeX = SCREEN.width * 0.05,
		sizeY = SCREEN.height * 0.05,
		-- text = "",
		-- fSize = 22,
		imageData = Images["icon_swap_priorities"],
		callback = function()
			Game:swapWorkerJobPriorities( 2, 3 )
		end,
		screenId = "game",
		parentId = "widGameWorkerJobPriorities",
		enabled = true,
	}
	
	-- switch 3rd and 4th
	Button:create{
		id = "btnGameSwapWorkerJobPriorities3",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.4,
		shape = "rect",
		sizeX = SCREEN.width * 0.05,
		sizeY = SCREEN.height * 0.05,
		-- text = "",
		-- fSize = 22,
		imageData = Images["icon_swap_priorities"],
		callback = function()
			Game:swapWorkerJobPriorities( 3, 4 )
		end,
		screenId = "game",
		parentId = "widGameWorkerJobPriorities",
		enabled = true,
	}
	
	-- switch 4th and 5th
	Button:create{
		id = "btnGameSwapWorkerJobPriorities4",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.5,
		shape = "rect",
		sizeX = SCREEN.width * 0.05,
		sizeY = SCREEN.height * 0.05,
		-- text = "",
		-- fSize = 22,
		imageData = Images["icon_swap_priorities"],
		callback = function()
			Game:swapWorkerJobPriorities( 4, 5 )
		end,
		screenId = "game",
		parentId = "widGameWorkerJobPriorities",
		enabled = true,
	}
	
	-- switch 5th and 6th
	Button:create{
		id = "btnGameSwapWorkerJobPriorities5",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.6,
		shape = "rect",
		sizeX = SCREEN.width * 0.05,
		sizeY = SCREEN.height * 0.05,
		-- text = "",
		-- fSize = 22,
		imageData = Images["icon_swap_priorities"],
		callback = function()
			Game:swapWorkerJobPriorities( 5, 6 )
		end,
		screenId = "game",
		parentId = "widGameWorkerJobPriorities",
		enabled = true,
	}
	
	-- switch 6th and 7th
	Button:create{
		id = "btnGameSwapWorkerJobPriorities6",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.7,
		shape = "rect",
		sizeX = SCREEN.width * 0.05,
		sizeY = SCREEN.height * 0.05,
		-- text = "",
		-- fSize = 22,
		imageData = Images["icon_swap_priorities"],
		callback = function()
			Game:swapWorkerJobPriorities( 6, 7 )
		end,
		screenId = "game",
		parentId = "widGameWorkerJobPriorities",
		enabled = true,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- SHORT INFO MESSAGE VIEW
	NotificationView:create{
		id = "novGameShortMessages",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.95,
		sizeX = SCREEN.width * 0.4,
		sizeY = SCREEN.height * 0.1,
		screenId = "game",
		enabled = true,
		-- hideWhenEmpty = true,
		bgColor = COLORS.black,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- MINIMAP (RADAR)
	MiniMap:create{
		id = "gameMinimap",
		x = SCREEN.width * 0.3,
		y = SCREEN.height * 0.3,
		sizeX = SCREEN.width * 0.4,
		sizeY = SCREEN.height * 0.4,
		screenId = "game",
		enabled = true,
		visible = false,
	}
	
	-- OPEN/HIDE MINIMAP
	Button:create{
		id = "btnGameToggleMinimap",
		x = SCREEN.width * 0.05,
		y = SCREEN.height * 0.15,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_minimap"],
		callback = function()
			if Game.isPaused then return end
			GUImgr:toggleVisibility( "gameMinimap" )
			GUImgr:setVisibility( "widGameConfirmDestroyBuilding", false )
		end,
		screenId = "game",
		enabled = true,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- OPEN PAUSE MENU
	Button:create{
		id = "btnGameOpenPauseMenu",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.95,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_settings"],
		callback = function()
			Game:setPaused( true )
			GUImgr:setVisibility( "widGamePauseMenu", true )
			GUImgr:setVisibility( "btnGameOpenPauseMenu", false )
			GUImgr:setVisibility( "btnGameOpenMissionBriefing", false )
		end,
		screenId = "game",
		enabled = true,
	}
	
	-- PAUSE MENU
	Widget:create{
		id = "widGamePauseMenu",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.5,
		sizeY = SCREEN.height * 0.6,
		screenId = "game",
		visible = false,
	}
	
	-- CONTINUE GAME
	Button:create{
		id = "btnGameResumeGame",
		x = SCREEN.width * 0.4,
		y = SCREEN.height * 0.75,
		shape = "rect",
		sizeX = SCREEN.width * 0.19,
		sizeY = SCREEN.height * 0.09,
		text = getString( "game_resume" ),
		fSize = 22,
		callback = function()
			Game:setPaused( false )
			GUImgr:setVisibility( "widGamePauseMenu", false )
			GUImgr:setVisibility( "btnGameOpenPauseMenu", true )
			GUImgr:setVisibility( "btnGameOpenMissionBriefing", true )
		end,
		screenId = "game",
		parentId = "widGamePauseMenu",
		enabled = true,
	}
	
	-- OPTION: CHANGE GAMESPEED
	Label:create{
		id = "lblGameOptionGamespeed",
		x = SCREEN.width * 0.4,
		y = SCREEN.height * 0.3,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = getString( "game_gamespeed" ),
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Label:create{
		id = "lblGameOptionGamespeedValue",
		x = SCREEN.width * 0.65,
		y = SCREEN.height * 0.3,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = "-",
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Slider:create{
		id = "sliGameOptionGamespeed",
		x = SCREEN.width * 0.55,
		y = SCREEN.height * 0.3,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.05,
		minValue = 0.5,
		maxValue = 2.0,
		value = 1.0,
		step = 0.1,	-- only allow discrete values
		horizontal = true,
		roundCorners = true,
		onValueChanged = function( value )
			GUImgr:getObjectById( "lblGameOptionGamespeedValue" ):setText( string.format( "%.1fx", value ) )
			Game:guiActionSetGamespeed( value )
		end,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	-- OPTION: SOUND VOLUME
	Label:create{
		id = "lblGameOptionSoundVolume",
		x = SCREEN.width * 0.4,
		y = SCREEN.height * 0.4,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = getString( "game_sound_volume" ),
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Label:create{
		id = "lblGameOptionSoundVolumeValue",
		x = SCREEN.width * 0.65,
		y = SCREEN.height * 0.4,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = "-",
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Slider:create{
		id = "sliGameOptionSoundVolume",
		x = SCREEN.width * 0.55,
		y = SCREEN.height * 0.4,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.05,
		minValue = 0,
		maxValue = 1,
		value = 0.8,
		step = 0.1,	-- only allow discrete values
		horizontal = true,
		roundCorners = true,
		onValueChanged = function( value )
			if value > 0 then
				GUImgr:getObjectById( "lblGameOptionSoundVolumeValue" ):setText( string.format( "%.1f", value ) )
			else
				GUImgr:getObjectById( "lblGameOptionSoundVolumeValue" ):setText( getString( "game_option_off" ) )
			end
		end,
		screenId = "game",
		parentId = "widGamePauseMenu",
		enabled = false,	-- for now! TODO
	}
	
	-- OPTION: MUSIC VOLUME
	Label:create{
		id = "lblGameOptionMusicVolume",
		x = SCREEN.width * 0.4,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = getString( "game_music_volume" ),
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Label:create{
		id = "lblGameOptionMusicVolumeValue",
		x = SCREEN.width * 0.65,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = "-",
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Slider:create{
		id = "sliGameOptionMusicVolume",
		x = SCREEN.width * 0.55,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.05,
		minValue = 0,
		maxValue = 1,
		value = 0.4,
		step = 0.1,	-- only allow discrete values
		horizontal = true,
		roundCorners = true,
		onValueChanged = function( value )
			if value > 0 then
				GUImgr:getObjectById( "lblGameOptionMusicVolumeValue" ):setText( string.format( "%.1f", value ) )
			else
				GUImgr:getObjectById( "lblGameOptionMusicVolumeValue" ):setText( getString( "game_option_off" ) )
			end
			Music:setMasterVolume( value )
		end,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	-- OPTION: SCROLL SPEED (CAMERA PAN)
	Label:create{
		id = "lblGameOptionScrollSpeed",
		x = SCREEN.width * 0.4,
		y = SCREEN.height * 0.6,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = getString( "game_scroll_speed" ),
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Label:create{
		id = "lblGameOptionScrollSpeedValue",
		x = SCREEN.width * 0.65,
		y = SCREEN.height * 0.6,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = "-",
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Slider:create{
		id = "sliGameOptionScrollSpeed",
		x = SCREEN.width * 0.55,
		y = SCREEN.height * 0.6,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.05,
		minValue = 0.4,
		maxValue = 2,
		value = 1.0,
		step = 0.2,	-- only allow discrete values
		horizontal = true,
		roundCorners = true,
		onValueChanged = function( value )
			GUImgr:getObjectById( "lblGameOptionScrollSpeedValue" ):setText( string.format( "%.1fx", value ) )
			Game:guiActionSetScrollSpeed( value )
		end,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- MISSION BRIEFING WIDGET
	Widget:create{
		id = "widGameMissionBriefing",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.5,
		sizeY = SCREEN.height * 0.6,
		screenId = "game",
		visible = false,
	}
	
	-- MISSION BRIEFING CAPTION
	Label:create{
		id = "lblGameMissionBriefingCaption",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		text = getString( "game_mission" ),
		fSize = 40,
		screenId = "game",
		parentId = "widGameMissionBriefing",
	}
	
	-- MISSION BRIEFING TEXT
	TextScroll:create{
		id = "txtGameMissionBriefing",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.4,
		sizeY = SCREEN.height * 0.4,
		screenId = "game",
		parentId = "widGameMissionBriefing",
		text = "",
		fSize = 22,
	}
	
	-- OPEN MISSION BRIEFING
	Button:create{
		id = "btnGameOpenMissionBriefing",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.95,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		text = getString( "game_mission" ),
		fSize = 22,
		callback = function()
			Game:openMissionBriefingWindow()
			-- Game:setPaused( true )
			-- GUImgr:setVisibility( "btnGameOpenMissionBriefing", false )
			-- GUImgr:setVisibility( "btnGameOpenPauseMenu", false )
			-- GUImgr:setVisibility( "widGameMissionBriefing", true )
		end,
		screenId = "game",
		enabled = true,
	}
	
	-- CLOSE MISSION BRIEFING
	Button:create{
		id = "btnGameCloseMissionBriefing",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.75,
		shape = "rect",
		sizeX = SCREEN.width * 0.19,
		sizeY = SCREEN.height * 0.09,
		text = getString( "game_resume" ),
		fSize = 22,
		callback = function()
			Game:setPaused( false )
			GUImgr:setVisibility( "btnGameOpenMissionBriefing", true )
			GUImgr:setVisibility( "btnGameOpenPauseMenu", true )
			GUImgr:setVisibility( "widGameMissionBriefing", false )
			GUImgr:getObjectById( "btnGameCloseMissionBriefing" ):setText( getString( "game_resume" ) )
		end,
		screenId = "game",
		parentId = "widGameMissionBriefing",
		enabled = true,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- CONFIRM QUIT GAME
	Widget:create{
		id = "widGameConfirmQuitGame",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.2,
		screenId = "game",
		visible = false,
	}
	
	-- GOTO TO MAIN MENU CONFIRMATION WIDGET
	Button:create{
		id = "btnGameOpenQuitGameWidget",
		x = SCREEN.width * 0.6,
		y = SCREEN.height * 0.75,
		shape = "rect",
		sizeX = SCREEN.width * 0.19,
		sizeY = SCREEN.height * 0.09,
		text = getString( "game_quit" ),
		fSize = 22,
		callback = function()
			GUImgr:setVisibility( "widGamePauseMenu", false )
			GUImgr:setVisibility( "widGameConfirmQuitGame", true )
		end,
		screenId = "game",
		parentId = "widGamePauseMenu",
		enabled = true,
	}
	
	-- QUIT CONFIRMATION TEXT
	Label:create{
		id = "lblGameConfirmQuitGame",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.45,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		text = getString ( "game_confirm_cancel_mission" ),
		fSize = 32,
		screenId = "game",
		parentId = "widGameConfirmQuitGame",
	}
	
	-- QUIT TO MAIN MENU
	Button:create{
		id = "btnGameQuit",
		x = SCREEN.width * 0.45,
		y = SCREEN.height * 0.55,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = "Ja",
		-- fSize = 22,
		imageData = Images["icon_yes"],
		callback = function()
			GUImgr:setVisibility( "btnGameOpenPauseMenu", true )
			GUImgr:setVisibility( "btnGameOpenMissionBriefing", true )
			GUImgr:setVisibility( "widGamePauseMenu", false )
			GUImgr:setVisibility( "widGameConfirmQuitGame", false )
			Game:close()
			-- MainMenu:open()
			-- PostGame:open( game.gamestate )
		end,
		screenId = "game",
		parentId = "widGameConfirmQuitGame",
		enabled = true,
	}
	
	-- DO NOT QUIT TO MAIN MENU
	Button:create{
		id = "btnGameCancelQuitGame",
		x = SCREEN.width * 0.55,
		y = SCREEN.height * 0.55,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = "Nein",
		-- fSize = 22,
		imageData = Images["icon_no"],
		callback = function()
			GUImgr:setVisibility( "widGameConfirmQuitGame", false )
			GUImgr:setVisibility( "widGamePauseMenu", true )
		end,
		screenId = "game",
		parentId = "widGameConfirmQuitGame",
		enabled = true,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- CURRENT FPS
	Label:create{
		id = "lblGameFps",
		x = SCREEN.width * 0.01,
		y = SCREEN.height * 0.99,
		sizeX = SCREEN.width * 0.02,
		sizeY = SCREEN.height * 0.02,
		text = "",
		-- fSize = 10,
		screenId = "game",
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- Mainloop:add( game )
	GUImgr:addScreen( game )
	
	return game
end


return GameScreen
