
-- ! SET DEFAULT LANGUAGE HERE !
-- ! this value is only used for initialization !
-- ! value can be either "en" or "de" !
local defaultLang = "en"

local lg = love.graphics

TILESIZE = 125	-- ?

ENTITIES = {
	hardRock = 1,
	solidRock = 2,
	looseRock = 3,
	debris = 4,
	-- buildingArea = ?,
	toolStore = 5,
	rockRaider = 6,
	ore = 7,
	teleportPad = 8,
	energyCrystal = 9,
	constructionSite = 10,
	hoverScout = 11,
	digger = 12,
	constructionFence = 13,
	transportTruck = 14,
	robotBay = 15,
	vehicleFactory = 16,
	landslide = 17,
	teslaTower = 18,
	fledder = 19,
	teslaBolt = 20,
	waterGunTower = 21,
	probeDroid = 22,
	mobileLaserCutter = 23,
	supportStation = 24,
	carl = 25,
	rockSample = 26,
}

ENTITYCATEGORY = {
	wall = 1,
	debris = 2,
	building = 3,
	buildingArea = 4,
	buildingOther = 5,
	rockRaider = 6,
	resource = 7,
	vehicle = 8,
	landslide = 9,
	monster = 10,
	projectile = 11,
	robot = 12,
}

COLORS = {
	white = { 1, 1, 1 },
	black = { 0, 0, 0 },
	gray = { 0.8, 0.8, 0.8 },
	darkgray = { 0.4, 0.4, 0.4 },
	lightgray = { 0.9, 0.9, 0.9 },
	brown = { 0.2, 0.1, 0.0 },
	lightBrown = { 0.7, 0.6, 0.0 },
	blue = { 0, 0, 1 },
	beige = { 0.5, 0.5, 0 },
	orange = { 1, 0.5, 0 },
	green = { 0.1, 0.7, 0.5 },
	lightgreen = { 0.2, 1, 0.2 },
	yellow = { 1, 1, 0 },
	red = { 1, 0, 0 },
	hb_black = { 0, 0, 0, 0.8 },	-- healthbar background (trans black)
	hb_green = { 0.33, 1, 0, 0.8 },	-- healthbar light green
}

COSTS = {
	-- building costs
	[ENTITIES.toolStore] = { [ENTITIES.ore] = 5 },
	[ENTITIES.teleportPad] = { [ENTITIES.ore] = 8, [ENTITIES.energyCrystal] = 1 },
	[ENTITIES.robotBay] = { [ENTITIES.ore] = 15, [ENTITIES.energyCrystal] = 2 },
	[ENTITIES.vehicleFactory] = { [ENTITIES.ore] = 12, [ENTITIES.energyCrystal] = 2 },
	[ENTITIES.teslaTower] = { [ENTITIES.ore] = 15, [ENTITIES.energyCrystal] = 2 },
	[ENTITIES.waterGunTower] = { [ENTITIES.ore] = 15, [ENTITIES.energyCrystal] = 2 },
	[ENTITIES.supportStation] = { [ENTITIES.ore] = 12, [ENTITIES.energyCrystal] = 2 },
	
	-- unit costs
	[ENTITIES.digger] = { [ENTITIES.energyCrystal] = 1 },
	[ENTITIES.hoverScout] = { [ENTITIES.energyCrystal] = 1 },
	[ENTITIES.transportTruck] = { [ENTITIES.energyCrystal] = 1 },
	[ENTITIES.probeDroid] = { [ENTITIES.energyCrystal] = 1 },
	
	-- upgrade costs
	-- maybe like this?
	["up1_teleportPad"] = { [ENTITIES.ore] = 5 },
	
	-- research costs
	
}

--> images must be rendered in studio with an angle of 30°

----------------------------------------------------------------------------------------------------

function getIconByEntityType( entityType )
	
	local iconByEntityType = {
		[ENTITIES.toolStore] = Images["icon_tool_store"],
		[ENTITIES.teleportPad] = Images["icon_teleport_pad"],
		[ENTITIES.robotBay] = Images["icon_robot_bay"],
		[ENTITIES.teslaTower] = Images["icon_tesla_tower"],
		[ENTITIES.waterGunTower] = Images["icon_water_gun_tower"],
		[ENTITIES.supportStation] = Images["icon_support_station"],
		[ENTITIES.vehicleFactory] = Images["icon_vehicle_factory"],
	}
	return iconByEntityType[entityType]
	
end

----------------------------------------------------------------------------------------------------

function love.load()
	
	math.randomseed( os.time() )	-- to be replaced by RNGs?
	
	love.filesystem.setIdentity( "ROCK RAIDERS" )
	
	SCREEN = { height = lg.getHeight(), width = lg.getWidth() }
	
	WINDOW_HAS_FOCUS = true
	
	Tools				= require( "utility.tools" )
	Queue				= require( "utility.queue" )
	Images			= require( "utility.imageloader" )
	Pathfinding	= require( "utility.a_star" )
	Animation		= require( "utility.animation" )
	
	-- make sure the language is selected properly
	-- should be (string) "de" or "en"
	Lang = initLanguage()
	if not Lang then
		love.window.showMessageBox( "Language - Sprache",
			"Failed to init language! Sprache konnte nicht geladen werden!", "error" )
		love.event.quit()
		return
	end
	-- one file per language
	Strings			= require( "strings." .. Lang )
	
	-- keybinds
	Keybinds		= require( "keybinds" ):init()
	
	local Loop	= require( "utility.gameloop" )
	Mainloop		= Loop:create()	-- application logic, including GUI
	Gameloop		= Loop:create()	-- game updates only (such as entities) - TODO: move into game?
	
	Logger			= require( "utility.logger" ):create( "latest_log" )
	Simplejob		= require( "utility.simplejob" ):create()		-- actually a "manager" for jobs
	Camera			= require( "utility.camera" ):create()
	Renderer		= require( "renderer" ):create()
	Terrain			= require( "terrain" )
	Blocking		= require( "blocking" )
	Particles		= require( "utility.particlemanager" ):create()
	FogOfWar		= require( "fogofwar" )
	Health			= require( "health_component" )
	MobSpawner	= require( "mobspawner" )
	Music				= require( "music" ):init()
	
	OhwConstructionSite	= require( "overhead.overhead_construction_site" )
	OhwEntitySelection	= require( "overhead.overhead_entity_selection" )
	OhwWorkerTask				= require( "overhead.overhead_worker_task" )
	
	HardRock					= require( "entities.world.hard_rock" )
	SolidRock					= require( "entities.world.solid_rock" )
	LooseRock					= require( "entities.world.loose_rock" )
	BuildingArea			= require( "entities.buildings.building_area" )
	Debris						= require( "entities.world.debris" )
	Landslide					= require( "entities.world.landslide" )
	Building					= require( "entities.buildings.building" )	-- "template"
	Vehicle						= require( "entities.vehicles.vehicle" )	-- "template"
	ConstructionSite	= require( "entities.buildings.construction_site" )
	ConstructionFence	= require( "entities.buildings.construction_fence" )
	ToolStore					= require( "entities.buildings.tool_store" )
	TeleportPad				= require( "entities.buildings.teleport_pad" )
	RobotBay					= require( "entities.buildings.robot_bay" )
	VehicleFactory		= require( "entities.buildings.vehicle_factory" )
	RockRaider				= require( "entities.rock_raider" )
	Ore								= require( "entities.world.ore" )
	EnergyCrystal			= require( "entities.world.energy_crystal" )
	HoverScout				= require( "entities.vehicles.hover_scout" )
	Digger						= require( "entities.vehicles.digger" )
	TransportTruck		= require( "entities.vehicles.transport_truck" )
	TeslaTower				= require( "entities.buildings.tesla_tower" )
	Fledder						= require( "entities.mobs.fledder" )
	TeslaBolt					= require( "entities.other.tesla_bolt" )	-- "projectile"
	WaterGunTower			= require( "entities.buildings.water_gun_tower" )
	ProbeDroid				= require( "entities.robots.probe_droid" )
	MobileLaserCutter	= require( "entities.vehicles.mobile_laser_cutter" )
	SupportStation		= require( "entities.buildings.support_station" )
	Carl							= require( "entities.mobs.carl" )
	RockSample				= require( "entities.world.rock_sample" )
	
	EntityManager = require( "utility.entitymanager" ):create()
	MapLoader		= require( "utility.maploader" ):create()
	
	Mouse				= require( "mouse" ):create()
	UserInput		= require( "userinput" ):init()
	
	UIRenderer	= require( "ui.uirenderer" ):create()
	GUImgr			= require( "ui.guimgr" ):create()
	Tooltip			= require( "ui.tooltip" )
	Button			= require( "ui.button" )
	TextScroll	= require( "ui.textscroll" )	-- like a label but suited for more text + vertical scroll
	Widget			= require( "ui.widget" )
	Listselect	= require( "ui.listselect" )
	ValuePicker	= require( "ui.valuepicker" )
	NotificationView	= require( "ui.notificationview" )
	MiniMap			= require( "ui.minimap" )
	Label				= require( "ui.label" )
	Slider			= require( "ui.slider" )
	
	Game				= require( "game" ):create()	-- logic
	
	GameScreen	= require( "ingame" ):create()	-- screen
	MapEditor		= require( "map_editor" ):create()
	MainMenu		= require( "main_menu" ):create()
	PostGame		= require( "post_game_screen" ):create()
	Credits			= require( "credits" ):create()
	MainMenu:open()
	
	-- CameraTransform:translate( 200, 200 )
	-- CameraTransform:scale( 0.8 )
	
	-- Camera:setPosition( 250, 500 )
	Camera:setScale( 1 )
	
	-- play any of these songs
	-- Music:playAny{ "vgm-atmos-thevoid", "vgm-atmos-airlock" }
	-- or with sound volumes:
	-- Music:playAny( { "vgm-atmos-thevoid", "vgm-atmos-airlock" }, {1, 0.1} )
	
end

----------------------------------------------------------------------------------------------------

function love.update( dt )
	if dt > 0.3 then
		print( string.format( "warning: large dt: %.2f", dt ) )
		return
	end
	-- maybe consider to also do not allow dt of > 0.1 and maybe reduce it to a "normal" value?
	-- to avoid entities jumping around...
	if dt > 0.1 then
		dt = 0.016
	end
	Mainloop:update( dt )
end

----------------------------------------------------------------------------------------------------

function love.draw()
	
	Renderer:draw()
	UIRenderer:draw()
	
	-- debug - center of the screen
	-- lg.points( SCREEN.width * 0.5, SCREEN.height * 0.5 )
	
	-- debug: draw a grid for easier orientation and alignment
	-- lg.setColor( 1, 1, 1, 0.4 )
	-- local w = SCREEN.width
	-- local h = SCREEN.height
	-- local dx = w * 0.1
	-- local dy = h * 0.1
	-- for y = 1, 9 do
		-- lg.line( 0,y*dy, w, y*dy )
	-- end
	-- for x = 1, 9 do
		-- lg.line( x*dx,0, x*dx,h )
	-- end
	
end


function convertTo2D( x, y )
	return (2*y+x)/2, (2*y-x)/2
end

function convertToDimetric( x, y )
	return x-y, (x+y)/2
end

-- pass in a world position
-- return the (full) tile x and y index
function getTileIndex( x, y )
	local tileSize = 125
	-- return math.floor( x / tileSize ),	math.floor( y / tileSize )
	-- note: center of the top most tile is (0,0)
	return Tools.round( x / tileSize )+1,	Tools.round( y / tileSize )+1
end

-- pass in a world position
-- return x and y mini tile index
function getMiniTileIndex( x, y )
	local x = x + 62.5
	local y = y + 62.5
	local tileSize = 25
	-- important: middle mini tile is in the center of the tile
	return math.floor( x / tileSize )+1,	math.floor( y / tileSize )+1
	-- return Tools.round( x / tileSize )+1,	Tools.round( y / tileSize )+1
end

-- get the world coordinates from the mini tile center
function miniTileIndexToPosition( mtx, mty )
	local tileSize = 25
	-- local centerOffset = 13
	return (mtx-3) * tileSize, (mty-3) * tileSize
end

-- get the world coordinates from the full tile center
function tileIndexToPosition( xi, yi )
	return (xi-1) * TILESIZE, (yi-1) * TILESIZE
end

----------------------------------------------------------------------------------------------------

function Quit()
	Logger:log( "Terminating." )
	Logger:destroy()
	love.event.quit()
end

----------------------------------------------------------------------------------------------------

-- USERINPUT

function love.keypressed( key, scancode, isrepeat )
	UserInput:keyPressed( key, scancode, isrepeat )
end

function love.keyreleased( key, scancode )
	UserInput:keyReleased( key, scancode )
end


function love.wheelmoved(x, y)
	Mouse:wheelMoved(x, y)
end

function love.mousepressed( x, y, button, istouch, presses )
	Mouse:pressed( x, y, button, istouch, presses )
end

function love.mousereleased( x, y, button, istouch, presses )
	Mouse:released( x, y, button, istouch, presses )
end

function love.mousemoved( x, y, dx, dy, istouch )
	Mouse:moved( x, y, dx, dy )
end

----------------------------------------------------------------------------------------------------

-- Callback function triggered when window receives or loses mouse focus
-- note: also set to false when the mouse is over the window title bar
function love.mousefocus( hasFocus )
	WINDOW_HAS_FOCUS = hasFocus
end

----------------------------------------------------------------------------------------------------

function initLanguage()
	local fileInfo = love.filesystem.getInfo( "lang.lua", "file" )
	if fileInfo then
		local language = require( "lang" )
		print( "Found language file. Language is:", language )
		return language	-- "de" or "en"
	else
		print( "Language file missing." )
		local success, language = writeLanguageFile()
		if success then
			return language
		else
			-- failed to fetch the language
			return false
		end
	end
end


function writeLanguageFile()
	local content =
	"-- Define the langue of the ingame texts.\r\n"..
	"-- Legt die Sprache der Texte im Spiel fest.\r\n"..
	"-- English: \"en\"\r\n"..
	"-- Deutsch: \"de\"\r\n"..
	"return \""..defaultLang.."\"\r\n"
	
	print( "Writing language file ..." )
	local file = love.filesystem.newFile( "lang.lua" )
	local open_ok, open_error = file:open( "w" )
	if not open_ok then
		print( open_error )
		return false
	end
	local write_ok, write_error = file:write( content )
	if write_ok then
		print( "... done." )
	else
		print( write_error )
	end
	file:close()
	return write_ok, defaultLang
end

----------------------------------------------------------------------------------------------------

function getString( key )
	if not Strings[key] then
		print( "getString error - unknown key:", key )
		return string.format("?%s?", key)
	end
	return Strings[key]
end

----------------------------------------------------------------------------------------------------

