local MobSpawner = {}

function MobSpawner:create( game )
	local spawner = {}
	
	-- spawner.game = game	-- reference to the game...?
	spawner.enabled = true
	spawner.spawnTick = 0
	spawner.spawnFrequency = 1	-- default value, time between spawns
	spawner.mobLimit = 16				-- see setSpawnLimit
	
	spawner.spawnInfo = {}
	spawner.spawnInfo[ENTITIES.fledder] = { chance = 0.1, batchSizeMin = 1, batchSizeMax = 4, }
	spawner.spawnInfo[ENTITIES.carl] = { chance = 0.1, batchSizeMin = 1, batchSizeMax = 2, }
	
	----------------------------------------------------------------------------------------------------
	
	function spawner:enableMobSpawning()
		self.enabled = true
	end
	
	
	function spawner:disableMobSpawning()
		self.enabled = false
	end
	
	
	function spawner:setSpawnFrequency( freq )
		self.spawnFrequency = freq
	end
	
	
	-- if this number is reached, no new mobs will be spawned
	-- however, during a spawn iteration this limit can be exceeded
	-- (check is performed at the start of the spawn iteration)
	function spawner:setSpawnLimit( limit )
		self.mobLimit = limit
	end
	
	
	function spawner:setMobSpawnChance( mobType, chance )
		if not self.spawnInfo[mobType] then
			print( "mobspawner:setMobSpawnChance warning - no such mob type!", mobType )
			return false
		end
		local chance = Tools.clamp( chance, 0, 1 )
		self.spawnInfo[mobType].chance = chance
		return chance
	end
	
	----------------------------------------------------------------------------------------------------
	
	function spawner:tick( dt )
		
		-- preliminary checks
		if not self.enabled then
			return
		end
		
		self.spawnTick = self.spawnTick + dt
		if self.spawnTick > self.spawnFrequency then
			self.spawnTick = self.spawnTick - self.spawnFrequency
			self:spawnIteration()
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function spawner:spawnIteration()
		
		-- make sure there are not too many mobs!
		local monsters = EntityManager:getEntitiesFilter( function(e)
			return e.entityCategory == ENTITYCATEGORY.monster
		end )
		if #monsters >= self.mobLimit then
			return
		end
		
		-- mob spawning...
		
		local random = math.random
		
		-- note that 0;0 is the center of the first tile
		-- TODO: maybe 0;0 should actually be the very corner of the map?
		local minX, minY = 1 - 0.5*TILESIZE, 1 - 0.5*TILESIZE
		local maxX, maxY = Game.mapSizeX * TILESIZE - 0.5*TILESIZE, Game.mapSizeY * TILESIZE - 0.5*TILESIZE
		
		-- spawnable mob types
		for mobType, mobSpawnInfo in pairs( self.spawnInfo ) do
			if mobSpawnInfo.chance > 0 and random() < mobSpawnInfo.chance then
				-- TODO: condition check can be different for different mobs!
				local x = random( minX, maxX )
				local y = random( minY, maxY )
				local xi, yi = getTileIndex( x, y )
				local mini_xi, mini_yi = getMiniTileIndex( x, y )
				if not Game.fow:isTileExplored( xi, yi )
				and not Game.blocking:isMiniTileBlocked( mini_xi, mini_yi ) then
					local amount = 1
					if mobSpawnInfo.batchSizeMin and mobSpawnInfo.batchSizeMax then
						amount = random( mobSpawnInfo.batchSizeMin, mobSpawnInfo.batchSizeMax )
					end
					for i = 1, amount do
						EntityManager:createEntity( mobType, { x = x, y = y } )
					end
				end
			end
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function spawner:destroy()
		Gameloop:remove( self )
		self = nil
	end
	
	----------------------------------------------------------------------------------------------------
	
	Gameloop:add( spawner )
	
	return spawner
end

return MobSpawner
