local Mouse = {}


function Mouse:create()
	local mouse = {}
	
	mouse.updateRate = 0.01
	mouse.updateTick = 0
	mouse.pressedPos = { x = 0, y = 0 }
	
	-- events where listeners can be registered
	mouse.listeners = {}
	mouse.listeners["mousePressed"] = {}
	mouse.listeners["mouseReleased"] = {}
	mouse.listeners["mouseMoved"] = {}
	mouse.listeners["wheelMoved"] = {}
	
	
	function mouse:addListener( event, listener )
		if not self.listeners[event] then
			print( "mouse error: cannot add listener for invalid event: " )
			print( event )
			return false
		end
		
		table.insert( self.listeners[event], listener )
		return true
	end
	
	function mouse:removeListener( event, listener )
		if not self.listeners[event] then
			print( "mouse error: cannot remove listener for invalid event: " )
			print( event )
			return false
		end
		
		for i, f in pairs( self.listeners[event] ) do
			if f == listener then
				table.remove( self.listeners[event], i )
				print( "mouse: removed listener function" )
				return true
			end
		end
		print( "mouse warning: could not remove listener" )
		return false
	end
	
	
	function mouse:tick(dt)
		
		self.updateTick = self.updateTick + dt
		
		if self.updateTick > self.updateRate then
			
			self.updateTick = 0
		end
		
	end
	
	
	function mouse:getPosition()
		local mx, my = love.mouse.getPosition()
		return { x = mx, y = my }
	end
	
	
	-- get the world coordinates from the local mouse position
	-- see camera:getWorldPosition
	-- function mouse:getWorldPosition()
		-- return Camera:getWorldPosition( self:getPosition() )
	-- end
	
	
	function mouse:pressed( x, y, button, istouch, presses )
		
		-- inform the guimgr about the click event
		--> when the guimgr triggers some callback (e.g. a button click event) then do not process this click event any further here...
		if GUImgr:onMouseClick( x, y, button ) then
			--print( "mouse: guimgr catched mouse click", button )
			return
		end
		
		if button == 1 then	-- left click
			
			-- store the position where the mouse press started (for GUI interactions)
			self.pressedPos = {x = x, y = y}
			
			
		elseif button == 2 then	-- right click
			
			
		elseif button == 3 then	-- middle mouse button pressed
			
			
		end
		
		-- call all listener functions
		for i, listener in pairs( self.listeners.mousePressed ) do
			if listener.onMousePressed then
				listener:onMousePressed( x, y, button )
			else
				print( "mouse warning: missing listener function: onMousePressed" )
			end
		end
		
	end
	
	-- changing from one screen to another
	-- maybe reset cursor type... at least forget about the last mouse pos?
	function mouse:onScreenChanged(_state)
		
	end
	

	-- x: Amount of horizontal mouse wheel movement. Positive values indicate movement to the right.
	-- y: Amount of vertical mouse wheel movement. Positive values indicate upward movement.
	function mouse:wheelMoved(x, y)
		local keydown = love.keyboard.isDown
		--if keydown("lctrl") or keydown("rctrl") then
		-- if not guimgr:isMouseHoveringGuiElement() then
			-- if y > 0 then
				-- Camera:zoomIn()
			-- elseif y < 0 then
				-- Camera:zoomOut()
			-- end
		-- end
		
		-- call all listener functions
		for i, listener in pairs( self.listeners.wheelMoved ) do
			if listener.onWheelMoved then
				listener:onWheelMoved( x, y )
			else
				print( "mouse warning: missing listener function: onWheelMoved" )
			end
		end
		
	end
	
	
	function mouse:released( x, y, button, istouch, presses )
		
		GUImgr:onMouseReleased(x, y, button)
		
		-- call all listener functions
		for i, listener in pairs( self.listeners.mouseReleased ) do
			if listener.onMouseReleased then
				listener:onMouseReleased( x, y, button )
			else
				print( "mouse warning: missing listener function: onMouseReleased" )
			end
		end
		
	end
	
	
	function mouse:moved( x, y, dx, dy )
		
		-- call all listener functions
		for i, listener in pairs( self.listeners.mouseMoved ) do
			if listener.onMouseMoved then
				listener:onMouseMoved( dx, dy )
			else
				print( "mouse warning: missing listener function: onMouseMoved" )
			end
		end
		
	end
	
	
	Mainloop:add(mouse)
	
	return mouse
end


return Mouse
