local Music = {}
local audio = love.audio
local dir = "music/"


function Music:init()
	local music = {}
	
	music.sources = {}
	music.currentSong = nil
	music.masterVolume = 1
	-- music.timeBetweenSongs = 2
	-- music.waitBeforeNext = 0
	
	music["menu"] = {
		"vgm-atmos-airlock",
		"vgm-atmos-basss",
		"vgm-atmos-breached",
		"vgm-atmos-handcannon",
		"vgm-atmospheric-deepspace",
		"vgm-atmospheric-frigate",
	}
	music["game"] = {
		"vgm-atmos-breached",
		"vgm-atmos-handcannon",
		"vgm-atmospheric-deadspace",
		"vgm-atmospheric-deepspace",
		"vgm-atmospheric-frigate",
		"vgm-atmos-thevoid",
	}
	music["action"] = {
		"vgm-atmos-countdown",
		"vgm-atmos-hightension",
		"vgm-atmospheric-below",
	}
	
	music.setToUse = "menu"
	
	----------------------------------------------------------------------------------------------------
	
	function music:addSourceFile( id, filePath )
		if self.sources[id] then
			print( "music:addSourceFile - id already in use!", id, filePath )
			return false
		end
		local source = audio.newSource( filePath, "stream" )
		self.sources[id] = source
		-- print( "music:addSourceFile - new source:", id )
		return source
	end
	
	----------------------------------------------------------------------------------------------------
	
	function music:readMusicFromFolder( dir, ext )
		local ext = ext or "mp3"
		local extLength = string.len( ext )
		local musicFiles = love.filesystem.getDirectoryItems( dir )
		local n = 0
		for k, fileNameExt in ipairs( musicFiles ) do
			if string.sub( fileNameExt, -(extLength+1) ) == "."..ext then
				-- remove the file extension
				local fileName = string.sub( fileNameExt, 1, -(extLength+2) )
				-- print( "music: found file:", fileName )
				self:addSourceFile( fileName, dir..fileNameExt )
				n = n + 1
			end
		end
		-- print( "music:readMusicFromFolder - #files:", n )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function music:setMasterVolume( volume )
		if volume < 0 or volume > 1 then
			print( "music:setMasterVolume error - volume must be >0 and <1" )
			return false
		end
		-- print( "music:setMasterVolume", volume)
		self.masterVolume = volume
		self:setEnabled( volume > 0 )
		
		if self.masterVolume > 0 then
			local source = self.currentSong
			if source and source:isPlaying() then
				-- source:setVolume( source:getVolume() * self.masterVolume )
				source:setVolume( self.masterVolume )
			end
		end
		return self.masterVolume
	end
	
	----------------------------------------------------------------------------------------------------
	
	function music:setEnabled( flag )
		-- no change
		if self.enabled == flag then
			return
		end
		
		self.enabled = flag
		if self.enabled then
			-- play the next song
			local set = self[self.setToUse]
			if set then self:playAny( set ) end
		else
			-- stop the current song
			local source = self.currentSong
			if source then
				source:stop()
				self.currentSong = nil
			end
		end
	end
	
	----------------------------------------------------------------------------------------------------
	
	function music:play( id, vol )
		local source = self.sources[id]
		if not source then
			print( "music:play - unknown song id!", id )
			return false
		end
		-- reset volume (it becomes 0 after switching the set)
		local volume = 1
		if vol and vol >= 0 and vol <= 1 then
			volume = vol
		end
		source:setVolume( volume * self.masterVolume )
		source:play()
		self.currentSong = source
		-- print( "music: currently playing", id )
		-- print( "set:", self.setToUse )
		return source
	end
	
	
	function music:playAny( list, volumes )
		local i = math.random( 1, #list )
		local vol = 1
		if volumes then
			vol = volumes[i]
		end
		return self:play( list[i], vol )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function music:switchSetToUse( newSet )
		-- do not switch to the same set
		if self.setToUse == newSet then
			return
		end
		
		self.setToUse = newSet
		-- print( "music:switchSetToUse", newSet )
		
		-- set a flag so that the current song will be switched
		self.doSwitchSet = true
	end
	
	----------------------------------------------------------------------------------------------------
	
	function music:tick( dt )
		
		local source = self.currentSong
		
		if not source or not source:isPlaying() then
			
			-- play the next song
			local set = self[self.setToUse]
			self:playAny( set )
			
		else
			-- source is currently playing
			if self.doSwitchSet then
				local vol = source:getVolume()
				local newVolume = vol - dt
				-- note: it does not work properly when comparing against 0
				-- maybe a float precision issue?
				if vol > 0.01 then
					source:setVolume( newVolume )
					-- print( "music: decrease song volume", vol )
				else
					source:stop()
					self.currentSong = nil
					-- print( "music: stop current song" )
					
					-- do not forget to reset the flag!
					self.doSwitchSet = false
				end
			end
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function music:destroy()
		Mainloop:remove( self )
		self = nil
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- music:addSourceFile( "ambiente1", dir.."vgm-atmos-airlock.mp3" )
	music:readMusicFromFolder( dir )
	
	Mainloop:add( music )
	
	return music
end


return Music
