local OhwConstructionSite = {}
local lg = love.graphics

local resources = {
	[ENTITIES.ore] = { imgData = Images["icon_collect_ore"] },
	[ENTITIES.energyCrystal] = { imgData = Images["icon_collect_energy_crystal"] },
}

function OhwConstructionSite:create( data )
	local widget = {}
	
	widget.x = data.x
	widget.y = data.y
	widget.visible = true
	widget.icon = data.icon	-- from the building
	
	widget.missingResourcesTotal = Tools.copyTable( data.resources )
	widget.missingResourcesNow = Tools.copyTable( data.resources )
	
	widget.rendererLayer = "overhead"
	
	----------------------------------------------------------------------------------------------------
	
	function widget:updateMissingResources( resources )
		self.missingResourcesNow = Tools.copyTable( resources )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function widget:setVisible( visibility )
		self.visible = visibility
	end
	
	----------------------------------------------------------------------------------------------------
	
	function widget:draw()
		if not self.visible then return end
		
		local x, y = convertToDimetric( self.x, self.y )
		
		-- images and text, should be large enough for the text...
		local size = 40
		
		-- resource image and text
		local i = 0
		for resType, data in pairs( resources ) do
			-- if self.missingResourcesNow[resType] and self.missingResourcesNow[resType] > 0 then
			if self.missingResourcesNow[resType] then
				local img = data.imgData
				local scale = size / img.image:getWidth()
				local total = self.missingResourcesTotal[resType]
				local have = total - self.missingResourcesNow[resType]
				local text = string.format( "%d/%d", have, total )
				lg.setColor( COLORS.white )
				lg.draw( img.image, x, y - i * size, 0, scale, scale, img.ox, img.oy )
				-- lg.circle( "line", x, y - i * size, 2 )
				lg.setColor( COLORS.black )
				lg.printf( text, x - 0.5 * size + 1, y - i * size + 1, size, "center" )
				lg.setColor( COLORS.white )
				lg.printf( text, x - 0.5 * size, y - i * size, size, "center" )
				i = i + 1
			end
		end
		
		-- building icon
		if self.icon then
			local y = y
			if i > 1 then
				y = y - (i-1) * 0.5 * size
			end
			local img = self.icon
			local scaleX = size / img.image:getWidth()
			local scaleY = size / img.image:getHeight()
			lg.draw( img.image, x - size, y, 0, scaleX, scaleY, img.ox, img.oy )
			lg.rectangle( "line", x - 1.5 * size, y - 0.5 * size, size, size, 4, 4 )
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function widget:destroy()
		Renderer:remove( self, self.rendererLayer )
		self = nil
	end
	
	----------------------------------------------------------------------------------------------------
	
	Renderer:add( widget, widget.rendererLayer )
	
	return widget
end

return OhwConstructionSite
