local OhwWorkerTask = {}
local lg = love.graphics

local size = 30


function OhwWorkerTask:create( data )
	local widget = {}
	
	widget.entityId = data.entityId
	widget.visible = true
	widget.icon = nil
	widget.offsetY = data.offsetY or 0
	
	local bubbles = Images["thought_bubbles"]
	widget.bubbles = bubbles
	widget.bubblesScaleX = size / bubbles.image:getWidth()
	widget.bubblesScaleY = size / bubbles.image:getHeight()
	
	widget.rendererLayer = "overhead"
	
	----------------------------------------------------------------------------------------------------
	
	function widget:setVisible( visibility )
		self.visible = visibility
	end
	
	----------------------------------------------------------------------------------------------------
	
	function widget:setImage( imageData )
		self.icon = imageData
	end
	
	----------------------------------------------------------------------------------------------------
	
	function widget:draw()
		if not self.visible then return end
		
		local entity = EntityManager:getEntityById( self.entityId )
		if not entity then return end
		
		local x, y = convertToDimetric( entity:getPosition() )
		y = y - self.offsetY
		
		-- white, a bit transparent
		local color = { 1, 1, 1, 0.6 }
		
		if self.icon then
			
			-- draw thought bubbles below the icon
			do
				local img = self.bubbles
				local sx = self.bubblesScaleX
				local sy = self.bubblesScaleY
				lg.setColor( color )
				lg.draw( img.image, x - size, y + size, 0, sx, sy, img.ox, img.oy )
			end
			
			local img = self.icon
			local sx = size / img.image:getWidth()
			local sy = size / img.image:getHeight()
			lg.setColor( color )
			lg.rectangle( "fill", x - 1.5 * size, y - 0.5 * size, size, size, 4, 4 )
			lg.setColor( COLORS.white )
			lg.draw( img.image, x - size, y, 0, sx, sy, img.ox, img.oy )
			
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function widget:destroy()
		Renderer:remove( self, self.rendererLayer )
		self = nil
	end
	
	----------------------------------------------------------------------------------------------------
	
	Renderer:add( widget, widget.rendererLayer )
	
	return widget
end


return OhwWorkerTask
