local Label = {}
local lg = love.graphics

function Label:create( data )
	local label = {}
	label.id = data.id
	label.z = 2
	
	label.screenId = data.screenId	-- parent screen (must be!)
	label.parentId = data.parentId	-- parent widget (if any)
	label.canHover = true
	
	label.x = data.x
	label.y = data.y
	label.sizeX = data.sizeX or SCREEN.width * 0.2
	label.sizeY = data.sizeY or SCREEN.height * 0.2
	
	if data.visible == nil then
		label.visible = true
	else
		label.visible = data.visible
	end
	
	label.maxFontSize = data.fSize or 20
	label.color = data.color or {1,1,1}	-- text color
	
	label.frame = data.frame
	label.fill = data.fill
	
	----------------------------------------------------------------------------------------------------
	
	function label:draw()
		if not self.visible then
			return
		end
		
		if GUImgr:getCurrentScreenId() ~= self.screenId then
			return
		end
		
		if self.parentId ~= nil
		and not (GUImgr:isEnabled(self.parentId) and GUImgr:isVisible(self.parentId)) then
			return
		end
		
		if self.fill then
			lg.setColor( 0.2, 0.2, 0.2, 0.5 )
			lg.rectangle( "fill", self.x - self.text:getWidth() * 0.5, self.y - self.text:getHeight() * 0.5, self.text:getWidth() * 1.2, self.text:getHeight() * 1.2 )
			lg.setColor( 1, 1, 1 )
		end
		
		if self.frame then
			lg.rectangle( "line", self.x - self.text:getWidth() * 0.5, self.y - self.text:getHeight() * 0.5, self.text:getWidth() * 1.2, self.text:getHeight() * 1.2 )
		end
		
		if self.color then
			--lg.setColor(self.color.r, self.color.g, self.color.b, self.color.a)
			lg.setColor( self.color )
		end
		lg.draw( self.text, self.x, self.y, 0, 1, 1, self.text:getWidth() * 0.5, self.text:getHeight() * 0.5 )
		
		lg.setColor( 1, 1, 1 )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function label:setText( text )
		local fSize = self.maxFontSize
		
		if self.text then
			self.text:set( text )
		else
			self.text = lg.newText( lg.newFont(fSize), text )
		end
		
		self:autoAdjustFontSize( self.text )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function label:autoAdjustFontSize( text, maxFontSize )
		local fSize = maxFontSize or self.maxFontSize
		while text:getWidth() >= self.sizeX
		or text:getHeight() >= self.sizeY do
			fSize = fSize - 2
			if fSize < 8 then
				print("label warning: UI element too small for text!")
				break
			end
			text:setFont( lg.newFont(fSize) )
		end
	end
	
	----------------------------------------------------------------------------------------------------
	
	if data.text then
		label:setText( data.text )
	end
	
	GUImgr:addObject( label )
	UIRenderer:add( label )
	
	return label
end

return Label
