local Tooltip = {}
local lg = love.graphics

-- TODO:
-- option to use a fixed position for the tooltip
-- colors for background and frame, not just the text itself

function Tooltip:create( data )
	local tooltip = {}
	tooltip.id = data.id
	tooltip.z = 5							-- place very high above all other GUI elements
	tooltip.element = data.element	-- "parent", e.g. a button
	tooltip.screenId = data.element.screenId
	tooltip.parentId = data.element.id
	
	tooltip.maxFontSize = data.maxFontSize or 20
	tooltip.color = data.color or { 1.0, 1.0, 1.0 }
	tooltip.alignMode = data.alignMode or "left"
	tooltip.width = data.width or 120
	tooltip.height = data.height or 60
	tooltip.maxTextWidth = 0.9 * tooltip.width
	tooltip.maxTextHeight = 0.9 * tooltip.height
	
	----------------------------------------------------------------------------------------------------
	
	function tooltip:setText( _text, alignMode )
		self.text = self.text or lg.newText( lg.newFont( self.maxFontSize ), "" )
		self.text:setf( _text or "", self.maxTextWidth, alignMode or self.alignMode or "left" )
		self:autoAdjustFontSize( self.text, self.maxTextWidth, self.maxTextHeight )
	end
	
	
	function tooltip:autoAdjustFontSize( text, sizeX, sizeY )
		local fSize = self.maxFontSize or 40
		
		while text:getWidth() >= sizeX
		or text:getHeight() >= sizeY do
			fSize = fSize - 2
			if fSize < 8 then
				print( "tooltip warning: UI element too small for text!", self.id )
				break
			end
			text:setFont( lg.newFont( fSize ) )
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function tooltip:draw()
		if not self.visible then
			return
		end
		
		if self.parentId ~= nil
		and not (GUImgr:isEnabled(self.parentId) and GUImgr:isVisible(self.parentId)) then
			return
		end
		
		local width, height = love.graphics.getDimensions()
		local mouseX, mouseY = love.mouse.getPosition()
		-- x and y = top left corner
		local x = Tools.clamp( mouseX, 0, width - self.width )
		local y = Tools.clamp( mouseY - self.height, 0, height - self.height )
		lg.setColor( 0.1, 0.1, 0.1, 0.7 )
		lg.rectangle( "fill", x, y, self.width, self.height )
		lg.setColor( 1, 1, 1 )
		lg.rectangle( "line", x, y, self.width, self.height )
		
		love.graphics.setColor( self.color )
		lg.draw( self.text, x + 0.5 * self.width, y + 0.5 * self.height, 0, 1, 1, self.maxTextWidth * 0.5, self.text:getHeight() * 0.5 )
		lg.setColor( 1, 1, 1 )
	end
	
	----------------------------------------------------------------------------------------------------
	
	tooltip:setText( data.text )
	
	UIRenderer:add( tooltip )
	
	return tooltip
end


return Tooltip
