local UIRenderer = {}
local lg = love.graphics

-- render GUI objects only

local sortFunction = function( a, b )
	return a.z < b.z
end


function UIRenderer:create()
	local renderer = {
		objects = {},
	}
	
	
	function renderer:add( obj )
		if not obj.screenId then
			print( "UIRenderer error: cannot add element without screenId!", obj.id )
			return false
		end
		if not obj.draw then
			print( "UIRenderer error: cannot add element without draw function!", obj.id )
			return false
		end
		if obj.visible == nil then obj.visible = true end
		self.objects[obj.screenId] = self.objects[obj.screenId] or {}
		table.insert( self.objects[obj.screenId], obj )
	end
	
	
	function renderer:remove( obj )
		for i, screen in pairs( self.objects ) do
			for k, v in pairs( screen ) do
				if v == obj then
					table.remove( self.objects[i], k )
					return true
				end
			end
		end
		return false
	end
	
	
	function renderer:draw()
		-- draw UI for the current screen only
		local screenId = GUImgr:getCurrentScreenId()
		if not self.objects[screenId] then
			return
		end
		
		-- sort everything from the current screen by their z value
		table.sort( self.objects[screenId], sortFunction )
		for i, obj in pairs( self.objects[screenId] ) do
			if obj.visible then
				lg.setColor( 1, 1, 1 )
				obj:draw()
			end
		end
	end
	
	
	return renderer
end


return UIRenderer
