local UserInput = {}


function UserInput:init()
	local input = {}
	
	-- observer pattern? kinda
	-- list of listener to be notified when certain events occur
	input.listeners = {}
	input.listeners["keyPressed"] = {}
	input.listeners["keyReleased"] = {}
	
	
	-- events are: keyPressed, keyReleased
	function input:addListener( event, listener )
		if not self.listeners[event] then
			print( "userinput error: cannot add listener for invalid event: " )
			print( event )
			return false
		end
		
		table.insert( self.listeners[event], listener )
		--print( "userinput: added listener!" )
		return true
	end
	
	function input:removeListener( event, listener )
		if not self.listeners[event] then
			print( "userinput error: cannot remove listener for invalid event: " )
			print( event )
			return false
		end
		
		for i, f in pairs( self.listeners[event] ) do
			if f == listener then
				table.remove( self.listeners[event], i )
				print( "userinput: removed listener function" )
				return true
			end
		end
		print( "userinput warning: could not remove listener" )
		return false
	end
	
	
	function input:keyPressed( key, scancode, isrepeat )
		
		-- call all listener functions
		for i, listener in pairs( self.listeners.keyPressed ) do
			if listener.onKeyPressed then
				listener:onKeyPressed( key, scancode )
			else
				print( "userinput warning: missing listener function: onKeyPressed" )
			end
		end
		
	end
	
	function input:keyReleased( key, scancode )
		
		-- call all listener functions
		for i, listener in pairs( self.listeners.keyReleased ) do
			if listener.onKeyReleased then
				listener:onKeyReleased( key, scancode )
			else
				print( "userinput warning: missing listener function: onKeyReleased" )
			end
		end
	end
	
	return input
end

return UserInput
