local Camera = {}
local lg = love.graphics
local minScale = 0.5--1
local maxScale = 4--2

function Camera:create()
	local camera = {
		transform = love.math.newTransform(),
		scale = 1,	-- equal scale in x and y
		x = 0,	-- offset
		y = 0,
		scrollSpeed = 1,
	}
	
	----------------------------------------------------------------------------------------------------
	
	function camera:set()
		lg.push()
		lg.applyTransform( self.transform )
	end
	
	function camera:unset()
		lg.pop()
	end
	
	----------------------------------------------------------------------------------------------------
	
	function camera:getTransform()
		return self.transform
	end
	
	function camera:updateTransform()
		self.transform:reset()
		self.transform:translate( -self.x, -self.y )
		self.transform:scale( 1/self.scale, 1/self.scale )
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- centers the camera so that the position is in the center of the SCREEN
	function camera:setPosition( x, y )
		local x, y = convertToDimetric( x, y )
		--print( "Camera:", x, y )
		self.x = x * (1/self.scale) - SCREEN.width / 2
		self.y = y * (1/self.scale) - SCREEN.height / 2
		--print("camera: ", self.x, self.y)
		
		self:updateTransform()
	end
	
	
	function camera:getPosition()
		local x = (self.x + SCREEN.width / 2) * self.scale
		local y = (self.y + SCREEN.height / 2) * self.scale
		return convertTo2D( x, y )
	end
	
	
	-- TODO?
	function camera:move( dx, dy )
		local mult = self.scrollSpeed
		self.x = self.x + dx * mult
		self.y = self.y + dy * mult
		
		self:updateTransform()
	end
	
	----------------------------------------------------------------------------------------------------
	
	function camera:getScale()
		return self.scale
	end
	
	function camera:setScale( factor )
		if not factor then
			factor = 1
		end
		
		-- remember old position (center)
		local x, y = self:getPosition()
		
		self.scale = Tools.clamp( factor, minScale, maxScale )
		-- set focus again
		self:setPosition( x, y )
		
		self:updateTransform()
	end
	
	function camera:zoomIn()
		local scale = self:getScale()
		self:setScale( scale - 0.1 )
	end
	
	function camera:zoomOut()
		local scale = self:getScale()
		self:setScale( scale + 0.1 )
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- check if the passed in position (world coordinates) is currently visible on the screen
	-- optional: allow distance to the edge (center of the entity can be off-screen, but the entity is partly on-screen)
	function camera:isWorldPositionOnScreen( x, y, tolerance )
		local width = SCREEN.width
		local height = SCREEN.height
		local tolerance = tolerance or SCREEN.width * 0.2
		local sx, sy = self:getTransform():transformPoint( convertToDimetric( x, y ) )
		return sx >= -tolerance and sx <= width+tolerance and sy >= -tolerance and sy <= height+tolerance
	end
	
	----------------------------------------------------------------------------------------------------
	
	camera:setPosition( 0, 0 )
	
	return camera
end

return Camera
