local entitymanager = {}

function entitymanager:create()
	local em = { entities = {}, entcat = {}, nextId = 1 }
	
	-------------------------------------------------------------------------------------------------------------------------------------------------
	
	-- wrapper to create new entities
	-- type = the entity type (number)
	-- entityData = table with all initial entity properties, i.e. a certain entityID if it is created on the client side
	function em:createEntity( eType, entityData )
		
		local newId = self:getNextEntityId()
		--print("->", newId)
		
		local entity
		
		if eType == ENTITIES.hardRock then
			entity = HardRock:create( entityData )
			
		elseif eType == ENTITIES.solidRock then
			entity = SolidRock:create( entityData )
			
		elseif eType == ENTITIES.looseRock then
			entity = LooseRock:create( entityData )
			
		elseif eType == ENTITIES.debris then
			entity = Debris:create( entityData )
			
		elseif eType == ENTITIES.landslide then
			entity = Landslide:create( entityData )
			
		elseif eType == ENTITIES.buildingArea then
			entity = BuildingArea:create( entityData )
			
		elseif eType == ENTITIES.toolStore then
			entity = ToolStore:create( entityData )
			
		elseif eType == ENTITIES.robotBay then
			entity = RobotBay:create( entityData )
			
		elseif eType == ENTITIES.vehicleFactory then
			entity = VehicleFactory:create( entityData )
			
		elseif eType == ENTITIES.rockRaider then
			entity = RockRaider:create( entityData )
			
		elseif eType == ENTITIES.ore then
			entity = Ore:create( entityData )
			
		elseif eType == ENTITIES.energyCrystal then
			entity = EnergyCrystal:create( entityData )
			
		elseif eType == ENTITIES.teleportPad then
			entity = TeleportPad:create( entityData )
			
		elseif eType == ENTITIES.constructionSite then
			entity = ConstructionSite:create( entityData )
			
		elseif eType == ENTITIES.hoverScout then
			entity = HoverScout:create( entityData )
			
		elseif eType == ENTITIES.digger then
			entity = Digger:create( entityData )
			
		elseif eType == ENTITIES.transportTruck then
			entity = TransportTruck:create( entityData )
			
		elseif eType == ENTITIES.constructionFence then
			entity = ConstructionFence:create( entityData )
			
		elseif eType == ENTITIES.teslaTower then
			entity = TeslaTower:create( entityData )
			
		elseif eType == ENTITIES.fledder then
			entity = Fledder:create( entityData )
			
		elseif eType == ENTITIES.teslaBolt then
			entity = TeslaBolt:create( entityData )
			
		elseif eType == ENTITIES.waterGunTower then
			entity = WaterGunTower:create( entityData )
			
		elseif eType == ENTITIES.probeDroid then
			entity = ProbeDroid:create( entityData )
			
		elseif eType == ENTITIES.mobileLaserCutter then
			entity = MobileLaserCutter:create( entityData )
			
		elseif eType == ENTITIES.supportStation then
			entity = SupportStation:create( entityData )
			
		elseif eType == ENTITIES.carl then
			entity = Carl:create( entityData )
			
		elseif eType == ENTITIES.rockSample then
			entity = RockSample:create( entityData )
			
		else
			print("Entity Manager: UNKNOWN entity type", eType)
		end
		
		
		if not entity then
			return false
		end
		
		if entity.load then
			entity:load( newId )
		end
		
		self:addEntity( entity )
		
		return entity
	end
	
	-------------------------------------------------------------------------------------------------------------------------------------------------
	
	function em:addEntity( entity )
		
		if not entity.id then
			return false
		end
		
		-- store all entity references in one big table, index = ID
		self.entities[entity.id] = entity
		
		-- additionally store the ID in another table, depending on the entity category
		local ec = entity.entityCategory
		-- make sure the category exists
		if not self.entcat[ec] then
			self.entcat[ec] = {}
		end
		
		-- insert the new ID
		table.insert( self.entcat[ec], entity.id )
		--print("entmgr: new entity", entity.entityType, "in category", ec)
		
		-- make sure we do not use this ID or a lower one again
		if self.nextId <= entity.id then
			self.nextId = entity.id + 1
		end
		
		-- if Game then
			-- Game:onEntityCreated( entity )
		-- end
		
		-- if Mapeditor then
			-- Mapeditor:onEntityCreated( entity )
		-- end
		
		return entity.id
	end
	
	-------------------------------------------------------------------------------------------------------------------------------------------------
	
	function em:getAllEntities()
		local e = {}
		for i = 1, self.nextId-1 do
			if self.entities[i] then
				table.insert( e, self.entities[i] )
			end
		end
		return e
	end
	
	-------------------------------------------------------------------------------------------------------------------------------------------------
	
	-- fetch all entities (references) from one entityCategory
	function em:getEntitiesInCategory(_ec)
		if not self.entcat[_ec] then
			-- maybe no entity in this category has been created yet
			return {}
		end
		
		local result = {}
		for i = 1, #self.entcat[_ec] do
			local entity = self:getEntityById(self.entcat[_ec][i])
			if entity then
				table.insert(result, entity)
			end
		end
		
		return result
	end
	
	-------------------------------------------------------------------------------------------------------------------------------------------------
	
	-- fetch entities of a certain entity type
	function em:getEntitiesOfType( eType )
		if not eType then
			print( "entity manager error: invalid entity type!" )
			return {}
		end
		
		local result = {}
		for i = 1, self.nextId-1 do
			local entity = self.entities[i]
			if entity and entity.entityType and entity.entityType == eType then
				table.insert( result, entity )
			end
		end
		
		return result
	end
	
	-------------------------------------------------------------------------------------------------------------------------------------------------
	
	function em:getNextEntityId()
		self.nextId = self.nextId + 1
		return self.nextId - 1
	end
	
	-------------------------------------------------------------------------------------------------------------------------------------------------
	
	function em:getEntityById(_id)
		if self.entities[_id] then
			return self.entities[_id]
		end
		return false	-- no entity found with this Id
	end
	
	-------------------------------------------------------------------------------------------------------------------------------------------------
	
	function em:remove( entityOrId )
		if type(entityOrId) == "number" then
			return self:removeById( entityOrId )
		elseif type(entityOrId) == "table" then
			if entityOrId.id then
				return self:removeById( entityOrId.id )
			end
		end
		return false
	end
	
	function em:removeById( entityId )
		if self.entities[entityId] then
			local entity = self.entities[entityId]
			if entity.destroy then
				entity:destroy()
			end
			-- do not use table.remove here
			self.entities[entityId] = nil
			-- entity = nil
			return true
		end
		return false
	end
	
	-------------------------------------------------------------------------------------------------------------------------------------------------
	
	function em:destroyAllEntities()
		for i = 1, self.nextId-1 do
			if self.entities[i] and self.entities[i].destroy then
				self.entities[i]:destroy()
				--self.entities[i] = false
			end
		end
		
		self.entities = {}
		self.entcat = {}
		self.nextId = 1
	end
	
	-------------------------------------------------------------------------------------------------------------------------------------------------
	
	function em:areEntitiesOfTypeInArea( eType, area, number )
		local found = 0
		
		for i = 1, self.nextId-1 do
			local entity = self.entities[i]
			if entity and entity.entityType == eType then
				if tools.isPointInsideShape( entity:getPosition(), area ) then
					found = found + 1
					if found >= number then
						return true
					end
				end
			end
		end
		
		return false
	end
	
	-------------------------------------------------------------------------------------------------------------------------------------------------
	
	-- check if the given entity table is valid
	function em:isValidEntity( entity )
		if not entity or not entity.id then
			return false
		end
		return self:isValidEntityId( entity.id )
		-- for i = 1, self.nextId-1 do
			-- if self.entities[i] == entity then
				-- return true
			-- end
		-- end
		-- return false
	end
	
	-- check if the given entity ID is valid
	function em:isValidEntityId( entityId )
		return self:getEntityById( entityId ) ~= false
	end
	
	-------------------------------------------------------------------------------------------------------------------------------------------------
	
	-- scan and return all entities using a given filter function
	function em:getEntitiesFilter( filterFunction )
		
		if not filterFunction then
			print( "entitymanager: invalid filter function" )
			return {}
		end
		
		local result = {}
		-- important: do not use #self.entities because this value might be 0 even if there are entities!
		for i = 1, self.nextId-1 do
			local entity = self.entities[i]
			if entity and filterFunction( entity ) then
				--print( "entity match", entity.id )
				table.insert( result, entity )
			end
		end
		
		return result
	end
	
	-------------------------------------------------------------------------------------------------------------------------------------------------
	
	-- remove the entire manager
	function em:destroy()
		self:destroyAllEntities()
		self = nil
		print( "entitymanager destroyed" )
	end
	
	-------------------------------------------------------------------------------------------------------------------------------------------------
	
	return em
end

return entitymanager
