local Logger = {}
-- local osTime = os.time
local osDate = os.date


function Logger:create( fileName )
	local logger = {
		writeRate = 1,	-- write to file every X seconds
		writeTick = 0,
		buffer = Queue:create(),
		fileName = fileName or "log",
	}
	
	logger.file = love.filesystem.newFile( logger.fileName..".txt" )
	
	-- file may already exist, overwrite
	local open_ok, open_err = logger.file:open( "w" )
	if not open_ok then
		print( open_err )
		return false
	end
	logger.file:write( string.format( "%s: %s\n", osDate("%X"), "Logger started." ) )
	logger.file:close()
	
	
	function logger:log( content )
		if content and type(content) == "string" then
			self.buffer:pushRight( string.format( "%s: %s", osDate("%X") , content ) )
		else
			print( "logger: failed to log:", content )
		end
	end
	
	
	function logger:tick( dt )
		self.writeTick = self.writeTick + dt
		if self.writeTick >= self.writeRate then
			self.writeTick = self.writeTick - self.writeRate
			self:flush()
		end
	end
	
	
	function logger:flush()
		local n = self.buffer:size()
		if n > 0 then
			local file = self.file
			local open_ok, open_err = file:open( "a" )	-- append
			
			if not open_ok then
				print( open_err )
				return false
			end
			
			for i = 1, n do
				file:write( string.format( "%s\n", self.buffer:popLeft() ) )
			end
			
			file:close()
		end
		
		
	end
	
	
	function logger:destroy()
		self:flush()
		Mainloop:remove( self )
		self = nil
	end
	
	
	Mainloop:add( logger )
	
	return logger
end

return Logger
