local MapLoader = {}

function MapLoader:create()
	local maploader = { maps = {} }
	
	----------------------------------------------------------------------------------------------------
	
	-- fetch all maps inside the maps folder
	-- store them with their file name as index (file names should be unique anyway)
	function maploader:loadMapsFromMapFolder()
		self.maps = {}
		
		-- since the folder with the name "maps" exists in both the game and save directory
		-- the function will return all files from the folders in both locations!
		local mapFiles = love.filesystem.getDirectoryItems( "/maps" )
		for k, fileNameExt in ipairs( mapFiles ) do
			if string.sub( fileNameExt, -4 ) == ".lua" then
				-- remove the file extension
				local fileName = string.sub( fileNameExt, 1, -5 )
				--print("maploader: found map...", fileName)
				
				local mapData = require( "maps/" .. fileName )
				if mapData then
					-- table.insert( self.maps, { data = mapData, fileName = fileName, } )
					self.maps[fileName] = mapData
				end
			end
		end
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- load a map with a given name
	function maploader:loadMap( name )
		-- local map = load( "maps." .. name )
		
		if self.maps[name] then
			return self.maps[name]
		end
		
		-- map not found? reload...
		-- print( "MapLoader info: map not listed yet, reload...", name )
		self:loadMapsFromMapFolder()
		
		-- ... and try again
		if self.maps[name] then
			return Tools.copyTable( self.maps[name] )
		end
		
		print( "MapLoader error: map does not exist!", name )
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- get a table containing all map FILE names of maps that have been loaded + the number of maps
	-- without file extension
	function maploader:getMapFileNames()
		local names, n = {}, 0
		for fileName, map in pairs( self.maps ) do
			table.insert( names, fileName )
			n = n + 1
		end
		return names, n
	end
	
	----------------------------------------------------------------------------------------------------
	
	return maploader
end

return MapLoader
