local Particlemanager = {}
local lg = love.graphics


function Particlemanager:create()
	local particles = {
		counter = 0,
		rendererLayer = "particles",
	}
	
	particles.ps = {}	-- particle systems
	
	
	----------------------------------------------------------------------------------------------------
	
	function particles:createWallBreakingParticles( _x, _y )
		local x, y = convertToDimetric( _x, _y )
		local img = Images["ore_gray"].image
		local particleSystem = {}
		particleSystem.system = lg.newParticleSystem( img, 16 )
		particleSystem.system:setEmitterLifetime( 0 )	-- required because it is -1 otherwise
		particleSystem.system:setParticleLifetime( 1, 1.5 )
		particleSystem.system:setSizes( 0.4, 0.5, 0.4 )
		particleSystem.system:setEmissionArea( "uniform", 80, 32, 0 )
		particleSystem.system:setLinearAcceleration( -8, 64, 8, 64 )	-- xmin, ymin, xmax, ymax
		particleSystem.system:setSpin( 0, 2 )	-- in rad per second
		particleSystem.system:emit( 16 )
		-- keep the "true" position in order to properly decide when to draw the particles
		particleSystem.system:setPosition( x, y )
		-- offset in screen coordinates
		-- make it look like it is coming from above
		particleSystem.offsetX = 0
		particleSystem.offsetY = -128
		self:add( particleSystem )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function particles:tick( dt )
		
		self.counter = self.counter + dt
		
		for i, ps in pairs( self.ps ) do
			local pMin, pMax = ps.system:getParticleLifetime()
			local lifetime = ps.system:getEmitterLifetime()
			-- also take the particle lifetime into consideration, not only the emitter's!
			-- emitter lifetime of -1 means it emitts forever -> do not remove
			if (ps.startTime + lifetime + pMax <= self.counter) and (lifetime > -1) then
				table.remove( self.ps, i )
			else
				ps.system:update(dt)
			end
		end
	end
	
	
	function particles:add( _ps )
		if not _ps.startTime then
			_ps.startTime = self.counter
		end
		table.insert( self.ps, _ps )
	end
	
	
	function particles:draw()
		for i, ps in pairs( self.ps ) do
			local x, y = ps.system:getPosition()
			local xo, yo = ps.offsetX or 0, ps.offsetY or 0
			local screenX, screenY = Camera:getTransform():transformPoint( x,y )
			-- only draw visible (on screen) particle systems
			if screenX > 0 and screenY > 0
			and screenX < SCREEN.width and screenY < SCREEN.height then
				lg.draw( ps.system, x+xo, y+yo )
				
				-- lg.circle( "line", x, y, 4 )
				-- print( screenX, screenY )
			end
		end
	end
	
	
	function particles:remove( _ps )
		for i, ps in pairs( self.ps ) do
			if ps == _ps then
				ps.system:setEmitterLifetime( 0 )
				--print("ps: stop emitter")
				return true
			end
		end
	end
	
	
	-- remove all current particle systems (eg. when a level is completed)
	function particles:clear()
		self.ps = {}
	end
	
	
	function particles:destroy()
		Gameloop:remove( self )
		Renderer:remove( self, self.rendererLayer )
		self = nil
	end
	
	Gameloop:add( particles )
	Renderer:add( particles, particles.rendererLayer )
	
	return particles
end

return Particlemanager
