local Queue = {}

-- simple double queue, where elements can be inserted and removed from the left and the right more efficiently
-- than using table.insert and table.remove(first) -> because removing the first element causes all other elements to be moved
-- see https://www.lua.org/pil/11.4.html

-- changes:
-- instead of using first = 0 and last = -1 I added one to each so the very first index is 1 and not 0
-- added queue:size method to check how many elements there are

function Queue:create()
	local queue = { first = 1, last = 0 }
	
	
	function queue:pushLeft( value )
		local first = self.first - 1
		self.first = first
		self[first] = value
	end
	
	function queue:pushRight( value )
		local last = self.last + 1
		self.last = last
		self[last] = value
	end
	
	
	function queue:popLeft()
		local first = self.first
		if first > self.last then
			-- list is empty
			return nil
		end
		local value = self[first]
		self[first] = nil	-- garbage collection
		self.first = first + 1
		return value
	end
	
	function queue:popRight()
		local last = self.last
		if self.first > last then
			-- list is empty
			return nil
		end
		local value = self[last]
		self[last] = nil	-- garbage collection
		self.last = last - 1
		return value
	end
	
	
	function queue:peekLeft()
		local first = self.first
		if first > self.last then
			-- list is empty
			return nil
		end
		return self[first]
	end
	
	function queue:peekRight()
		local last = self.last
		if self.first > last then
			-- list is empty
			return nil
		end
		return self[last]
	end
	
	-- how many items are stored in the queue?
	function queue:size()
		return self.last - self.first + 1
	end
	
	return queue
end

return Queue
