local Simplejobmanager = {}

function Simplejobmanager:create()
	local sjm = { jobs = {}, nextId = 1 }
	
	
	function sjm:addJob( data )
		local job = {
			id = self.nextId,
			callback = data.callback or function() end,
			params = data.params or {},
			active = true,
			intervall = data.intervall or 1,
			tick = 0,
			--limit = data.limit,	-- not yet implemented
		}
		table.insert( self.jobs, job )
		
		self.nextId = self.nextId + 1
		return job
	end
	
	
	function sjm:tick( dt )
		for i, job in pairs( self.jobs ) do
			if job.active then
				-- only tick while active
				job.tick = job.tick + dt
				
				if job.tick >= job.intervall then
					job.tick = job.tick - job.intervall
					
					-- execute the callback function
					-- with callback parameter table
					if job.callback( job.params ) == true then
						-- callback returns true = job is done, remove it
						table.remove( self.jobs, i )
					end
				end
			end
		end
	end
	
	
	function sjm:getJobById( id )
		for i, job in pairs( self.jobs ) do
			if job.id == id then
				return job
			end
		end
		return false
	end
	
	
	function sjm:remove( job_or_id )
		local job
		if type( job_or_id ) == "table" then
			job = job_or_id
		else
			job = self:getJobById( job_or_id )
		end
		
		if job then
			for i, j in pairs( self.jobs ) do
				if j.id == job.id then
					table.remove( self.jobs, i )
					return true
				end
			end
		else
			if job_or_id then
				print( "simplejob: job not found!", job_or_id )
			end
			return false
		end
	end
	
	
	Mainloop:add( sjm )
	
	return sjm
end

return Simplejobmanager
